---------------------------------------------------------------------
このファイルは、TiMidity++ の Tcl/Tk インタフェイス TkMidity について
説明しています。

TkMidity は、パネル表示により TiMidity++ 本体と対話的な動作を行うため
のインタフェースです。Tk を使用していますので、Motif のような美しい
表示画面を、Motif ライブラリ無しで利用することができます。

* WHAT'S NEW in 1.5

- timer callback を使用したトレース表示
- Forward / Backward ボタン
- たくさんのバグフィックス..


* CONTENTS

このアーカイブには以下のものが入っています:

README.tk	- このファイル
tk_c.c		- tk-interface のコントローラのソースコード
tkmidity.ptcl	- メインパネル (preprocess 前)
tkpanel.tcl	- メインパネル
browser.tcl	- ファイルブラウザ
misc.tcl	- 各種コード
tkbitmaps/*.xbm	- TkMidity 用ビットマップファイル

(訳注 : これはオリジナルの配布での内容です。TiMidity++ では、
全てのファイルがあらかじめ適切なディレクトリに入っています)


* USAGE

TkMidity には 4 つのモード (repeat , shuffle, auto-start, auto-exit)
があります。
"Repeat" では、指定ファイルが全て演奏された後、最初に戻り再び演奏を
開始します。
"Shuffle" では、指定ファイルの中からランダムに演奏するファイルを
選んで演奏します。
"Auto-start" では、TkMidity が起動すると同時に演奏を開始します。
"Auto-exit" では、全ての曲の演奏が終ったら自動的に TkMidity を終了
します。
いずれの設定も、"Save Config" メニューでセーブしておくことができます。

ディスプレー上の設定は "Display" メニューで変更することができます。
この設定も、"Save Config" メニューでセーブされ、次回起動時に設定が
再現されます。

ver.1.3 から、ファイルのオープン/クローズ のメニューと、キーボード
ショートカットがサポートされています。演奏する曲目に、任意のファイルを
追加できるようになっています。
また、キーボードショートカットは以下のようになっています:

[Enter]		: 演奏開始
[Space]		: 一時停止 / 演奏再開
[c]		: 停止
[q]		: TkMidity の終了
[p] or [Left]	: 前の曲
[n] or [Right]	: 次の曲
[v] or [Down]	: ボリューム下げ (5%)
[V] or [Up]	: ボリューム上げ (5%)
[F10]		: メニューモードへ
[Alt]+[Any]	: メニューの内容を選択

ver.1.4 から、トレース表示がサポートされました。各 MIDI チャンネル毎の
ボリュームやパンポットの動きをリアルタイムに見ることができます。この
表示を見るときは、TiMidity++ の起動オプションに適切なフラグを指定して
ください(詳細はマニュアルを参照)。
(訳注: `-itt' と t を後ろに付けます)


* PROGRAM NOTES

この version では、Tcl7.5 および Tk4.1 のライブラリが、それぞれ必要です。
古い version では wish を使用していましたが、現在の version では直接
ライブラリをリンクしています。
また、利用する際には、shared memory へのアクセス権が必要です。


* TROUBLE SHOOTING

+演奏する際、そのファイルが存在することを TiMidity++ に渡す前に
スクリプト側で確認していますが、稀に「ファイルが存在しません」のような
エラーが出ることがあります。

		Takashi Iwai	<iwai@dragon.mm.t.u-tokyo.ac.jp>
				<http://bahamut.mm.t.u-tokyo.ac.jp/~iwai/>

[テキストの翻訳は長野大介 <breeze_geo@geocities.co.jp> が行いました]
