#ifndef __libOggFLAC_dll_i_h__
#define __libOggFLAC_dll_i_h__

/* #define IGNORE_libOggFLAC_OggFLAC__StreamEncoderStateString  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__StreamDecoderStateString  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_new  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_delete  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_serial_number  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_verify  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_streamable_subset  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_mid_side_stereo  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_loose_mid_side_stereo  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_channels  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_bits_per_sample  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_sample_rate  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_blocksize  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_lpc_order  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_qlp_coeff_precision  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_escape_coding  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_exhaustive_model_search  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_min_residual_partition_order  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_residual_partition_order  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_rice_parameter_search_dist  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_total_samples_estimate  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_metadata  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_write_callback  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_client_data  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_state  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_FLAC_stream_encoder_state  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_state  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_error_stats  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_streamable_subset  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_mid_side_stereo  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_loose_mid_side_stereo  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_channels  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_bits_per_sample  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_sample_rate  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_blocksize  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_lpc_order  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_qlp_coeff_precision  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_escape_coding  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_exhaustive_model_search  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_min_residual_partition_order  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_residual_partition_order  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_rice_parameter_search_dist  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_total_samples_estimate  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_init  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_finish  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_encoder_process  /* */
/* #define IGNORE_libOggFLAC_OggFLAC__stream_encoder_process_interleaved  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_new  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_delete  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_read_callback  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_write_callback  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_callback  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_error_callback  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_client_data  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_serial_number  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_application  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_all  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_application  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_all  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_state  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_FLAC_stream_decoder_state  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channels  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channel_assignment  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_bits_per_sample  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_sample_rate  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_blocksize  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_init  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_finish  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_flush  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_reset  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_single  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_metadata  /* */
#define IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_stream  /* */

#endif  /* __libOggFLAC_dll_i_h__ */
