//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by w32g_res.rc
//
#define IDD_DIALOG_SEARCHBOX            102
#define IDD_DIALOG_VORBIS               103
#define IDD_DIALOG_GOGO                 104
#define IDD_PREF_PLAYER_EN              106
#define IDD_PREF_TIMIDITY1_EN           107
#define IDD_PREF_TIMIDITY2_EN           108
#define IDD_PREF_TIMIDITY3_EN           109
#define IDD_PREF_TIMIDITY4_EN           110
#define IDD_DIALOG_MAIN_EN              111
#define IDM_MENU_MAIN_EN                112
#define IDD_DIALOG_VORBIS_EN            113
#define IDD_DIALOG_GOGO_EN              114
#define IDD_DIALOG_ONE_LINE_EN          118
#define IDD_DIALOG_ONE_LINE             119
#define IDB_BITMAP_KEYBORD42x20         122
#define IDB_BITMAP_TRACER               125
#define IDD_PREF_SYN1                   127
#define IDD_PREF_SYN1_EN                128
#define IDD_DIALOG_PREF                 132
#define IDC_CHECK_SEACHDIRRECURSIVE     1001
#define IDC_CHECK_DOCWNDINDEPENDENT     1002
#define IDC_CHECK_INIFILE_AUTOSAVE      1003
#define IDD_DIALOG_WAVEFORM             1004
#define IDC_CHECK_DOCWNDAUTOPOPUP       1004
#define IDC_CHECKBOX_CHORUS_LEVEL       1005
#define IDC_CHECKBOX_GLOBAL_REVERB      1006
#define IDC_CHECKBOX_REVERB_LEVEL       1007
#define IDC_CHECKBOX_CH01               1008
#define IDC_CHECKBOX_SRCHORUS           1008
#define IDC_CHECKBOX_CH02               1009
#define IDC_CHECKBOX_RESONANCE          1009
#define IDC_CHECKBOX_CH03               1010
#define IDC_CHECKBOX_SF_LPF             1010
#define IDC_CHECKBOX_CH04               1011
#define IDC_CHECKBOX_DRUM_EFFECT        1011
#define IDC_CHECKBOX_CH05               1012
#define IDC_CHECKBOX_PURE_INTONATION    1012
#define IDC_CHECKBOX_LPF_DEF            1012
#define IDC_CHECKBOX_CH06               1013
#define IDC_CHECKBOX_INIT_MI            1013
#define IDC_CHECKBOX_MOD_ENV            1013
#define IDC_CHECKBOX_CH07               1014
#define IDC_CHECKBOX_PAN_DELAY          1014
#define IDC_CHECKBOX_CH08               1015
#define IDC_CHECKBOX_CH09               1016
#define IDC_CHECKBOX_CH10               1017
#define IDC_CHECKBOX_CH11               1018
#define IDC_CHECKBOX_CH12               1019
#define IDC_CHECKBOX_CH13               1020
#define IDC_CHECKBOX_CH14               1021
#define IDC_CHECKBOX_CH15               1022
#define IDC_CHECKBOX_CH16               1023
#define IDC_CHECKBOX_CH17               1024
#define IDC_CHECKBOX_CH18               1025
#define IDC_CHECKBOX_CH19               1026
#define IDC_CHECKBOX_CH20               1027
#define IDC_CHECKBOX_CH21               1028
#define IDC_CHECKBOX_CH22               1029
#define IDC_CHECKBOX_CH23               1030
#define IDC_CHECKBOX_CH24               1031
#define IDC_CHECKBOX_CH25               1032
#define IDC_CHECKBOX_CH26               1033
#define IDC_CHECKBOX_CH27               1034
#define IDC_CHECKBOX_CH28               1035
#define IDC_CHECKBOX_CH29               1036
#define IDC_CHECKBOX_CH30               1037
#define IDC_CHECKBOX_CH31               1038
#define IDC_CHECKBOX_CH32               1039
#define IDD_PREF_PLAYER                 1040
#define IDC_CHECKBOX_NOT_DRAG_START     1041
#define IDC_CHECKBOX_NOT_CONTINUE       1042
#define IDC_CHECKBOX_AUTOREFINE         1043
#define IDC_CHECKBOX_AUTOUNIQ           1044
#define IDC_CHECKBOX_AUTOQUIT           1045
#define IDC_GROUPBOX_PLAYERMODE         1046
#define IDC_RADIOBUTTON_JAPANESE        1047
#define IDC_RADIOBUTTON_ENGLISH         1048
#define IDC_GROUPBOX_LANGUAGE           1049
#define IDC_EDIT_CONFIG_FILE            1050
#define IDC_BUTTON_CONFIG_FILE          1051
#define IDD_PREF_TIMIDITY1              1052
#define IDC_BUTTON_CFG_EDIT             1052
#define IDD_PREF_TIMIDITY2              1053
#define IDC_CHECKBOX_REDUCE_VOICE       1054
#define IDC_CHECKBOX_SPECIAL_TONEBANK   1055
#define IDC_GROUPBOX_DEFAULT_MIDI_TYPE  1058
#define IDC_EDIT_OUTPUT_FILE            1059
#define IDC_CHECKBOX_DRUM_CHANNEL_MASK  1060
#define IDC_CHECKBOX_CTL_TRACE_PLAYING  1061
#define IDD_PREF_TIMIDITY3              1062
#define IDC_CHECKBOX_FORCE_KEYSIG       1062
#define IDC_BUTTON_4                    1063
#define IDC_CHECKBOX_AMP_COMPENSATION   1063
#define IDC_BUTTON_8                    1064
#define IDC_BUTTON_16                   1065
#define IDC_BUTTON_24                   1066
#define IDC_BUTTON_32                   1067
#define IDC_BUTTON_40                   1068
#define IDC_BUTTON_48                   1069
#define IDC_BUTTON_OUTPUT_FILE_DEL      1070
#define IDC_CHECKBOX_LINEAR             1071
#define IDC_CHECKBOX_BYTESWAP           1072
#define IDC_CHECKBOX_UNSIGNED           1073
#define IDC_CHECKBOX_SIGNED             1074
#define IDC_CHECKBOX_ALAW               1075
#define IDC_CHECKBOX_ULAW               1076
#define IDC_CHECKBOX_16BITS             1077
#define IDC_CHECKBOX_8BITS              1078
#define IDC_BUTTON_HIGH                 1079
#define IDC_BUTTON_MIDDLE               1080
#define IDC_BUTTON_LOW                  1081
#define IDC_BUTTON_OUTPUT_FILE          1082
#define IDC_RADIOBUTTON_WIN32AUDIO      1083
#define IDC_RADIOBUTTON_RIFF_WAVE       1084
#define IDC_RADIOBUTTON_LIST_MIDI_EVENT 1085
#define IDC_RADIOBUTTON_RAW_WAVEFORM    1086
#define IDC_RADIOBUTTON_SUN_AUDIO       1087
#define IDC_RADIOBUTTON_AIFF            1088
#define IDC_GROUPBOX_OUTPUT             1089
#define IDD_PREF_TIMIDITY4              1090
#define IDC_CHECKBOX_DRUM_CHANNEL       1091
#define IDC_EDIT_CONTROL_RATIO          1092
#define IDC_EDIT_MODIFY_RELEASE         1093
#define IDC_EDIT_AMPLIFICATION          1094
#define IDC_EDIT_ENV_ATTACK             1094
#define IDC_EDIT_CACHE_SIZE             1095
#define IDC_EDIT_AUTOMAXIMIZE           1095
#define IDC_EDIT_VELOCITYTABLE          1095
#define IDC_EDIT_KEY_ADJUST             1096
#define IDC_EDIT_SPECIAL_TONEBANK       1097
#define IDC_CHECKBOX_NOT_LOOPING        1097
#define IDC_EDIT_DEFAULT_TONEBANK       1098
#define IDC_CHECKBOX_AUTOSTART          1098
#define IDC_EDIT_REDUCE_VOICE           1099
#define IDC_CHECKBOX_RANDOM             1099
#define IDC_EDIT_DRUM_POWER             1099
#define IDC_EDIT_CTL_VEBOSITY           1100
#define IDC_EDIT_DELAY                  1101
#define IDC_RADIOBUTTON_DELAY_LEFT      1102
#define IDC_RADIOBUTTON_DELAY_RIGHT     1103
#define IDC_RADIOBUTTON_DELAY_CENTER    1104
#define IDC_CHECKBOX_TRACETEXT          1105
#define IDC_EDIT_REVERB                 1106
#define IDC_EDIT_CHORUS                 1107
#define IDC_BUTTON_UNIQ                 1108
#define IDC_BUTTON_REFINE               1109
#define IDC_BUTTON_CLEAR                1110
#define IDC_BUTTON_DOC                  1111
#define IDB_BITMAP_SLEEP                1112
#define IDM_MENU_MAIN                   1113
#define IDC_CHECKBOX_QUIET_CHANNEL      1114
#define IDM_MHTIMIDITY                  1115
#define IDDEFAULT                       1116
#define IDD_DIALOG_MAIN                 1117
#define IDC_EDIT_INFO                   1118
#define IDD_DIALOG_DEBUG                1119
#define IDC_BUTTON_VARIABLES_CHECK      1120
#define IDC_BUTTON_HEAP_CHECK           1121
#define IDC_BUTTON_EXITPROCESS          1122
#define IDC_BUTTON_EXIT                 1123
#define IDD_DIALOG_CONSOLE              1124
#define IDD_DIALOG_CONSOLE_EN           1125
#define IDD_DIALOG_TRACER               1126
#define IDD_DIALOG_DOC                  1127
#define IDD_DIALOG_DOC_EN               1128
#define IDD_DIALOG_SETTING              1129
#define IDD_DIALOG_SIMPLE_LIST          1130
#define IDD_DIALOG_SIMPLE_LIST_EN       1131
#define IDC_TOOLBARWINDOW_SUBWND        1132
#define IDC_TOOLBARWINDOW_MAIN          1133
#define IDC_RECT_PANEL                  1134
#define IDC_RECT_CANVAS                 1135
#define IDC_BUTTON_DEC                  1136
#define IDC_BUTTON_INC                  1137
#define IDC_EDIT_VERBOSITY              1138
#define IDC_BUTTON_VERBOSITY            1139
#define IDC_EDIT                        1140
#define IDD_DIALOG_WRD                  1141
#define IDD_DIALOG_SOUNDSPEC            1142
#define IDFLUSH                         1143
#define IDD_DIALOG_PREF_EN              1143
#define IDCLEAR                         1144
#define IDC_BUTTON_PREV                 1145
#define IDC_BUTTON_NEXT                 1146
#define IDC_CHECKBOX_VALID              1147
#define IDC_CHECKBOX_DEBUG_WND          1148
#define IDC_CHECKBOX_DEBUG_WND_VALID    1149
#define IDC_EDIT_FILENAME               1150
#define IDC_LISTBOX_PLAYLIST            1151
#define IDM_MFOPENFILE                  1152
#define IDM_MFOPENDIR                   1153
#define IDM_MFLOADPLAYLIST              1154
#define IDM_MFSAVEPLAYLISTAS            1155
#define IDM_MFEXIT                      1156
#define IDM_MCLOADINIFILE               1157
#define IDM_MCSAVEINIFILE               1158
#define IDM_MWPLAYLIST                  1159
#define IDM_MWTRACER                    1160
#define IDM_MWDOCUMENT                  1161
#define IDM_MWWRDTRACER                 1162
#define IDM_MWSOUNDSPEC                 1163
#define IDM_MWCONSOLE                   1164
#define IDM_MWDEBUG                     1165
#define IDM_MHTOPIC                     1166
#define IDM_MHHELP                      1167
#define IDM_MHVERSION                   1168
#define IDC_SCROLLBAR_PROGRESS          1169
#define IDC_SCROLLBAR_VOLUME            1170
#define IDI_ICON_TIMIDITY               1171
#define IDB_BITMAP_MAIN_BUTTON          1172
#define IDB_BITMAP_SUBWND_BUTTON        1173
#define IDC_CHECK_DOC                   1174
#define IDM_SETTING                     1175
#define IDC_CHECKBOX_CHORUS             1176
#define IDC_CHECKBOX_REVERB             1177
#define IDC_CHECKBOX_DELAY              1178
#define IDC_CHECKBOX_FREE_INST          1179
#define IDC_CHECKBOX_ANTIALIAS          1180
#define IDC_EDIT_VOICES                 1181
#define IDC_RADIO_16BITS                1182
#define IDC_RADIO_8BITS                 1183
#define IDC_RADIO_STEREO                1184
#define IDC_RADIO_MONO                  1185
#define IDC_EDIT_SAMPLE_RATE            1186
#define IDC_EDIT_NOISESHARPING          1187
#define IDC_CHECKBOX_MODWHEEL           1188
#define IDC_CHECKBOX_PORTAMENTO         1189
#define IDC_CHECKBOX_NRPNVIB            1190
#define IDC_CHECKBOX_CHPRESS            1191
#define IDC_CHECKBOX_OVOICE             1192
#define IDC_CHECKBOX_LOADINST_PLAYING   1193
#define IDC_EDIT_EFFECT_QUALITY         1193
#define IDC_RADIOBUTTON_GM              1194
#define IDC_RADIOBUTTON_GS              1195
#define IDC_RADIOBUTTON_XG              1196
#define IDC_EDIT1                       2000
#define IDC_EDIT2                       2001
#define IDC_EDIT_NUM                    2002
#define IDC_EDIT3                       2002
#define IDC_BUTTON_REVERSE              2003
#define IDC_COMBO_OUTPUT                2004
#define IDC_BUTTON_OUTPUT_OPTIONS       2005
#define IDC_COMBO_BANDWIDTH             2006
#define IDC_CHECK_DEFAULT               2007
#define IDC_COMBO_SAMPLE_RATE           2007
#define IDC_COMBO_MODE                  2008
#define IDC_CHECK_USE_TAG               2009
#define IDC_COMBO_OUTPUT_FORMAT         2013
#define IDC_CHECK_COMMANDLINE_OPTS      2015
#define IDC_EDIT_COMMANDLINE_OPTION     2016
#define IDC_COMBO_MPEG1_AUDIO_BITRATE   2017
#define IDC_COMBO_MPEG2_AUDIO_BITRATE   2018
#define IDC_COMBO_VBR_BITRATE_LOW       2019
#define IDC_COMBO_VBR_BITRATE_HIGH      2020
#define IDC_CHECK_16KHZ_LOW_PASS_FILTER 2021
#define IDC_CHECK_ENHANCED_LOW_PASS_FILTER 2022
#define IDC_EDIT_LPF_PARA1              2023
#define IDC_EDIT_LPF_PARA2              2024
#define IDC_COMBO_ENCODE_MODE           2025
#define IDC_CHECK_USEPSY                2026
#define IDC_CHECK_CPUMMX                2027
#define IDC_CHECK_CPUSSE                2028
#define IDC_CHECK_CPU3DNOW              2029
#define IDC_CHECK_CPUE3DNOW             2030
#define IDC_COMBO_EMPHASIS_TYPE         2031
#define IDC_CHECK_CPUCMOV               2032
#define IDC_COMBO_VBR                   2033
#define IDC_CHECK_VBR                   2034
#define IDC_CHECK_OUTFREQ               2035
#define IDC_EDIT_OUTFREQ                2036
#define IDC_CHECK_MSTHRESHOLD           2037
#define IDC_EDIT_MSTHRESHOLD_THRESHOLD  2038
#define IDC_EDIT_MSTHRESHOLD_MSPOWER    2039
#define IDC_CHECK_VERIFY                2040
#define IDC_BUTTON_APPLY                2041
#define IDC_COMBO_OUTPUT_MODE           2042
#define IDC_CHECK_USE_CPU_OPTS          2043
#define IDC_CHECK_CPUSSE2               2044
#define IDC_CHECK_MPEG1AUDIOBITRATE     2045
#define IDC_CHECK_MPEG2AUDIOBITRATE     2046
#define IDC_CHECK_VBR_BITRATE           2047
#define IDC_CHECK_OUTPUT_FORMAT         2048
#define IDC_CHECK_ENCODE_MODE           2049
#define IDC_CHECK_EMPHASIS_TYPE         2050
#define IDC_CHECK_CPUEMMX               2051
#define IDC_CHECKBOX_TVAA               2057
#define IDC_CHECKBOX_TVAD               2058
#define IDC_CHECKBOX_TVAR               2059
#define IDC_CHECKBOX_PDELAY             2060
#define IDC_CHECKBOX_EQ                 2061
#define IDC_CHECK_AUTOLOAD_PLAYLIST     2061
#define IDC_CHECKBOX_IEFFECT            2062
#define IDC_CHECK_AUTOSAVE_PLAYLIST     2062
#define IDC_STATIC_HEAD                 2063
#define IDC_STATIC_TAIL                 2064
#define IDC_BUTTON_1                    2065
#define IDC_BUTTON_2                    2066
#define IDC_BUTTON_3                    2067
#define IDC_EDIT_ONE_LINE               2068
#define IDC_EDIT_SUBWINDOW_MAX          2069
#define IDC_CHECK_POS_SIZE_SAVE         2070
#define IDC_COMBO_FORCE_KEYSIG          2072
#define IDC_COMBO_INIT_KEYSIG           2073
#define IDC_COMBO_IDPORT0               2085
#define IDC_COMBO_IDPORT1               2086
#define IDC_COMBO_IDPORT2               2087
#define IDC_COMBO_IDPORT3               2088
#define IDC_COMBO_SYN_THREAD_PRIORITY   2089
#define IDC_COMBO_PORT_NUM              2090
#define IDC_EDIT_SYN_SH_TIME            2091
#define IDC_COMBO_PROCESS_PRIORITY      2092
#define IDC_TAB_MAIN                    2093
#define IDC_CHECKBOX_TEMPER_EQUAL       2094
#define IDC_CHECKBOX_TEMPER_PYTHA       2095
#define IDC_BUTTON_CFG_RELOAD           2095
#define IDC_CHECKBOX_TEMPER_MEANTONE    2096
#define IDC_CHECKBOX_TEMPER_PUREINT     2097
#define IDC_CHECKBOX_TEMPER_USER0       2098
#define IDC_CHECKBOX_TEMPER_USER1       2099
#define IDC_CHECKBOX_TEMPER_USER2       2100
#define IDC_CHECKBOX_TEMPER_USER3       2101
#define IDC_COMBO_REVERB                2103
#define IDC_COMBO_MODULE                2104
#define IDC_CHECK_SYN_AUTOSTART         2104
#define IDC_COMBO_CHORUS                2105
#define IDC_COMBO_DELAY                 2106
#define IDC_COMBO_LPF                   2107
#define IDC_CHECKBOX_INIT_KEYSIG        2108
#define IDM_MHSUPPLEMENT                40002
#define IDM_FORCE_RELOAD                40003
#define IDM_MHONLINEHELP                40004
#define IDM_MHONLINEHELPCFG             40005
#define IDM_MHBTS                       40006
#define IDC_STATIC                      -1

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        157
#define _APS_NEXT_COMMAND_VALUE         40007
#define _APS_NEXT_CONTROL_VALUE         2109
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
