//Microsoft Developer Studio generated resource script.
//
#include "w32g_res.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#define APSTUDIO_HIDDEN_SYMBOLS
#include "windows.h"
#undef APSTUDIO_HIDDEN_SYMBOLS
#include "commctrl.h"
#include "w32g_rec.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// ﾆｭｰﾄﾗﾙ resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_NEU)
#ifdef _WIN32
LANGUAGE LANG_NEUTRAL, SUBLANG_NEUTRAL
#pragma code_page(932)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_DIALOG_MAIN DIALOGEX 0, 0, 308, 64
STYLE DS_MODALFRAME | DS_3DLOOK | DS_CENTER | WS_MINIMIZEBOX | WS_VISIBLE | 
    WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_DLGMODALFRAME | WS_EX_ACCEPTFILES | WS_EX_APPWINDOW
CAPTION "TiMidity++ Win32GUI"
MENU IDM_MENU_MAIN
FONT 8, "MS Sans Serif", 0, 0, 0x1
BEGIN
    CONTROL         "",IDC_RECT_PANEL,"Static",SS_GRAYFRAME | SS_SUNKEN | 
                    WS_TABSTOP,0,0,220,44
    CONTROL         "",IDC_RECT_CANVAS,"Static",SS_GRAYFRAME,220,0,68,44
    CONTROL         "ToolBarMain",IDC_TOOLBARWINDOW_MAIN,"ToolbarWindow32",
                    0x14c,2,46,112,17
    CONTROL         "ToolBarSubWnd",IDC_TOOLBARWINDOW_SUBWND,"ToolbarWindow32",
                    0x44,112,46,80,16
    SCROLLBAR       IDC_SCROLLBAR_PROGRESS,190,48,100,12,WS_DISABLED
    SCROLLBAR       IDC_SCROLLBAR_VOLUME,292,0,12,60,SBS_VERT
END

IDD_DIALOG_MAIN_EN DIALOGEX 0, 0, 308, 64
STYLE DS_MODALFRAME | DS_3DLOOK | DS_CENTER | WS_MINIMIZEBOX | WS_VISIBLE | 
    WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_DLGMODALFRAME | WS_EX_NOPARENTNOTIFY | WS_EX_ACCEPTFILES | 
    WS_EX_APPWINDOW
CAPTION "TiMidity++ Win32GUI"
MENU IDM_MENU_MAIN_EN
FONT 8, "MS Sans Serif", 0, 0, 0x1
BEGIN
    CONTROL         "",IDC_RECT_PANEL,"Static",SS_GRAYFRAME | SS_SUNKEN | 
                    WS_TABSTOP,0,0,220,44
    CONTROL         "",IDC_RECT_CANVAS,"Static",SS_GRAYFRAME,220,0,68,44
    CONTROL         "ToolBarMain",IDC_TOOLBARWINDOW_MAIN,"ToolbarWindow32",
                    0x14c,2,46,112,17
    CONTROL         "ToolBarSubWnd",IDC_TOOLBARWINDOW_SUBWND,"ToolbarWindow32",
                    0x44,112,46,80,16
    SCROLLBAR       IDC_SCROLLBAR_PROGRESS,190,48,100,12,WS_DISABLED
    SCROLLBAR       IDC_SCROLLBAR_VOLUME,292,0,12,60,SBS_VERT
END

IDD_DIALOG_DEBUG DIALOG DISCARDABLE  0, 0, 240, 240
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU
CAPTION "Debug Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman"
BEGIN
    PUSHBUTTON      "C L E A R",IDCLEAR,72,164,68,16,BS_CENTER
    CONTROL         "VALID",IDC_CHECKBOX_DEBUG_WND_VALID,"Button",
                    BS_AUTOCHECKBOX | BS_LEFTTEXT | WS_TABSTOP,20,164,36,16
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,16,164,44,16
    PUSHBUTTON      "E X I T",IDC_BUTTON_EXIT,144,164,48,16,BS_CENTER | 
                    WS_DISABLED
    PUSHBUTTON      "ExitProcess",IDC_BUTTON_EXITPROCESS,196,164,40,16,
                    BS_CENTER
    CONTROL         "",-1,"Static",SS_ETCHEDHORZ,0,184,240,1
    PUSHBUTTON      "Heaps Check",IDC_BUTTON_HEAP_CHECK,4,188,50,16,
                    BS_CENTER
    PUSHBUTTON      "Variables Check",IDC_BUTTON_VARIABLES_CHECK,60,188,50,
                    16,BS_CENTER
    EDITTEXT        IDC_EDIT,0,0,240,160,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | ES_WANTRETURN | 
                    WS_VSCROLL | WS_HSCROLL
END

IDD_DIALOG_CONSOLE_EN DIALOGEX 0, 0, 271, 181
STYLE DS_MODALFRAME | DS_3DLOOK | WS_MAXIMIZEBOX | WS_CAPTION | WS_SYSMENU
CAPTION "Console Window (TiMidity Win32GUI)"
FONT 8, "Times New Roman", 0, 0, 0x1
BEGIN
    PUSHBUTTON      "C L E A R",IDCLEAR,174,165,90,15,BS_CENTER
    EDITTEXT        IDC_EDIT_VERBOSITY,50,165,21,15,ES_OEMCONVERT
    CONTROL         "VALID",IDC_CHECKBOX_VALID,"Button",BS_AUTOCHECKBOX | 
                    BS_LEFTTEXT | WS_TABSTOP,126,165,32,15
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,5,165,96,16
    EDITTEXT        IDC_EDIT,0,0,270,160,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | ES_WANTRETURN | 
                    WS_VSCROLL | WS_HSCROLL,WS_EX_STATICEDGE
    PUSHBUTTON      "VERBOSITY",IDC_BUTTON_VERBOSITY,4,165,46,15,BS_CENTER,
                    WS_EX_CLIENTEDGE
    PUSHBUTTON      "<<",IDC_BUTTON_DEC,70,165,15,16,BS_CENTER
    PUSHBUTTON      ">>",IDC_BUTTON_INC,85,165,15,16,BS_CENTER
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,123,165,40,16
END

IDD_DIALOG_DOC_EN DIALOGEX 0, 0, 241, 180
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME
EXSTYLE WS_EX_ACCEPTFILES
CAPTION "Document Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman", 0, 0, 0x1
BEGIN
    EDITTEXT        IDC_EDIT_FILENAME,60,168,140,12,ES_CENTER | ES_READONLY
    PUSHBUTTON      "<<",IDC_BUTTON_PREV,204,168,16,12,BS_CENTER
    PUSHBUTTON      ">>",IDC_BUTTON_NEXT,220,168,16,12,BS_CENTER
    EDITTEXT        IDC_EDIT,0,0,236,160,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | ES_WANTRETURN | 
                    WS_VSCROLL | WS_HSCROLL
    EDITTEXT        IDC_EDIT_INFO,0,168,56,12,ES_READONLY
END

IDD_DIALOG_TRACER DIALOGEX 0, 0, 319, 221
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_ACCEPTFILES
CAPTION "Tracer Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman", 0, 0, 0x1
BEGIN
END

IDD_DIALOG_SIMPLE_LIST_EN DIALOGEX 0, 0, 242, 187
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME
EXSTYLE WS_EX_DLGMODALFRAME | WS_EX_ACCEPTFILES | WS_EX_CONTROLPARENT
CAPTION "List Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman", 0, 0, 0x1
BEGIN
    LISTBOX         IDC_LISTBOX_PLAYLIST,0,0,241,170,LBS_HASSTRINGS | 
                    LBS_WANTKEYBOARDINPUT | LBS_DISABLENOSCROLL | WS_VSCROLL | 
                    WS_HSCROLL | WS_GROUP | WS_TABSTOP
    PUSHBUTTON      "DOC",IDC_BUTTON_DOC,55,175,45,10,BS_CENTER
    PUSHBUTTON      "UNIQ",IDC_BUTTON_UNIQ,114,175,31,10,BS_CENTER
    PUSHBUTTON      "REFINE",IDC_BUTTON_REFINE,146,175,29,10,BS_CENTER
    PUSHBUTTON      "CLEAR",IDC_BUTTON_CLEAR,190,175,46,10,BS_CENTER | 
                    WS_GROUP
    EDITTEXT        IDC_EDIT_NUM,5,175,45,12,ES_READONLY
END

IDD_PREF_PLAYER_EN DIALOGEX 0, 0, 211, 187
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "Times New Roman"
BEGIN
    PUSHBUTTON      "Config File",IDC_BUTTON_CONFIG_FILE,3,11,52,11,
                    BS_CENTER
    EDITTEXT        IDC_EDIT_CONFIG_FILE,56,10,133,14,ES_AUTOHSCROLL | 
                    ES_OEMCONVERT
    GROUPBOX        "Player",IDC_GROUPBOX_PLAYERMODE,2,45,106,60,WS_GROUP
    CONTROL         "Not Continue",IDC_CHECKBOX_NOT_CONTINUE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,57,97,8
    CONTROL         "Not Drag Start",IDC_CHECKBOX_NOT_DRAG_START,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,69,97,8
    CONTROL         "Not Looping",IDC_CHECKBOX_NOT_LOOPING,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,81,97,8
    CONTROL         "Auto Start",IDC_CHECKBOX_AUTOSTART,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,93,97,8
    GROUPBOX        "PlayList",IDC_STATIC,114,45,84,64
    CONTROL         "Auto Quit",IDC_CHECKBOX_AUTOQUIT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,53,77,8
    CONTROL         "Auto Uniq",IDC_CHECKBOX_AUTOUNIQ,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,61,76,8
    CONTROL         "Auto Refine",IDC_CHECKBOX_AUTOREFINE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,69,75,8
    CONTROL         "Random Mode",IDC_CHECKBOX_RANDOM,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,77,75,8
    CONTROL         "Auto Load",IDC_CHECK_AUTOLOAD_PLAYLIST,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,89,75,8
    CONTROL         "Auto Save",IDC_CHECK_AUTOSAVE_PLAYLIST,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,118,97,75,8
    GROUPBOX        "Document",IDC_STATIC,6,109,68,38
    CONTROL         "Not Auto Display",IDC_CHECK_DOCWNDINDEPENDENT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,121,63,8
    CONTROL         "Popup",IDC_CHECK_DOCWNDAUTOPOPUP,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,133,59,10
    GROUPBOX        "Directory",IDC_STATIC,6,149,67,20
    CONTROL         "Recursive walk",IDC_CHECK_SEACHDIRRECURSIVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,10,157,57,10
    GROUPBOX        "Sub Window",IDC_STATIC,78,113,60,28
    LTEXT           "MAX",IDC_STATIC,82,125,18,8
    EDITTEXT        IDC_EDIT_SUBWINDOW_MAX,102,125,24,12,ES_CENTER,
                    WS_EX_RIGHT
    GROUPBOX        "INI File",IDC_STATIC,78,145,60,24
    CONTROL         "Auto Save",IDC_CHECK_INIFILE_AUTOSAVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,82,153,48,10
    GROUPBOX        "Language",IDC_GROUPBOX_LANGUAGE,142,113,56,28,WS_GROUP
    CONTROL         "Japanese",IDC_RADIOBUTTON_JAPANESE,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,146,121,42,8
    CONTROL         "English",IDC_RADIOBUTTON_ENGLISH,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,146,129,43,8
    GROUPBOX        "Window",IDC_STATIC,142,145,56,24
    CONTROL         "save pos,size",IDC_CHECK_POS_SIZE_SAVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,146,155,48,10
    PUSHBUTTON      "Reload Config File",IDC_BUTTON_CFG_RELOAD,3,29,73,12,
                    BS_MULTILINE
    PUSHBUTTON      "Edit",IDC_BUTTON_CFG_EDIT,191,11,19,11,BS_CENTER
END

IDD_PREF_TIMIDITY1_EN DIALOG DISCARDABLE  0, 0, 219, 173
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "Times New Roman"
BEGIN
    CONTROL         "Modulation Wheel",IDC_CHECKBOX_MODWHEEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,60,67,10
    CONTROL         "Portament",IDC_CHECKBOX_PORTAMENTO,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,70,45,10
    CONTROL         "NRPN Vibrato",IDC_CHECKBOX_NRPNVIB,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,100,57,10
    CONTROL         "MIDI Controllers",IDC_CHECKBOX_CHPRESS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,80,82,10
    CONTROL         "Trace Text Meta Event",IDC_CHECKBOX_TRACETEXT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,45,85,10
    CONTROL         "Overlapped same voce",IDC_CHECKBOX_OVOICE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,35,84,10
    CONTROL         "Attack Time Control",IDC_CHECKBOX_TVAA,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,110,74,10
    CONTROL         "Decay Time Control",IDC_CHECKBOX_TVAD,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,120,73,10
    CONTROL         "Release Time Control",IDC_CHECKBOX_TVAR,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,130,81,10
    CONTROL         "EQ Control",IDC_CHECKBOX_EQ,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,140,55,10
    CONTROL         "Insertion Effect Control",IDC_CHECKBOX_IEFFECT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,150,82,10
    CONTROL         "Static Chorus Level",IDC_CHECKBOX_CHORUS_LEVEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,12,70,10
    EDITTEXT        IDC_EDIT_CHORUS,184,11,24,12,ES_NUMBER
    CONTROL         "Static Reverb Level",IDC_CHECKBOX_REVERB_LEVEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,24,69,10
    EDITTEXT        IDC_EDIT_REVERB,184,23,24,12,ES_NUMBER
    CONTROL         "L&&R Delay",IDC_CHECKBOX_DELAY,"Button",BS_AUTOCHECKBOX | 
                    BS_LEFT | BS_TOP | WS_TABSTOP,104,88,55,10
    CONTROL         "Left",IDC_RADIOBUTTON_DELAY_LEFT,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,108,99,27,10
    CONTROL         "Right",IDC_RADIOBUTTON_DELAY_RIGHT,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,138,99,31,10
    CONTROL         "Rotate",IDC_RADIOBUTTON_DELAY_CENTER,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,171,99,38,10
    LTEXT           "Delay(msec)",IDC_STATIC,115,110,36,8,NOT WS_GROUP | 
                    WS_TABSTOP
    EDITTEXT        IDC_EDIT_DELAY,151,109,32,12,ES_NUMBER
    LTEXT           "Pseudo Reverb(0..1000)",IDC_STATIC,103,124,67,8,NOT 
                    WS_GROUP | WS_TABSTOP
    EDITTEXT        IDC_EDIT_MODIFY_RELEASE,184,122,28,12
    LTEXT           "Noise Shaping (0..6)",IDC_STATIC,104,67,57,8,NOT 
                    WS_GROUP | WS_TABSTOP
    EDITTEXT        IDC_EDIT_NOISESHARPING,165,65,28,12,ES_NUMBER
    CONTROL         "NRPN Drumpart Effect",IDC_CHECKBOX_DRUM_EFFECT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,90,82,10
    GROUPBOX        "GM/GS/XG",IDC_STATIC,1,3,93,160
    GROUPBOX        "Special",IDC_STATIC,100,3,116,132
    CONTROL         "Modulation Envelope",IDC_CHECKBOX_MOD_ENV,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,55,76,10
    COMBOBOX        IDC_COMBO_REVERB,6,12,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_CHORUS,6,24,84,54,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_DELAY,6,36,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_LPF,6,48,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_MODULE,106,148,103,78,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Emulate specific module",IDC_STATIC,100,138,116,25
    CONTROL         "Surround",IDC_CHECKBOX_PAN_DELAY,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,104,77,41,10
END

IDD_PREF_TIMIDITY2_EN DIALOG DISCARDABLE  0, 0, 223, 186
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "Times New Roman"
BEGIN
    GROUPBOX        "Play",IDC_STATIC,1,3,117,114
    LTEXT           "Maxmum Voices",IDC_STATIC,6,15,50,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_VOICES,63,14,24,12,ES_NUMBER
    LTEXT           "Amplification(%)",IDC_STATIC,6,31,53,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_AMPLIFICATION,63,29,24,12
    CONTROL         "Antialiasing instrument",IDC_CHECKBOX_ANTIALIAS,"Button",
                    BS_AUTOCHECKBOX | BS_LEFT | BS_TOP | WS_TABSTOP,6,46,78,
                    10
    CONTROL         "Load Instruments While Playing",
                    IDC_CHECKBOX_LOADINST_PLAYING,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,56,101,10
    CONTROL         "Unload Instruments between files",
                    IDC_CHECKBOX_FREE_INST,"Button",BS_AUTOCHECKBOX | 
                    BS_LEFT | BS_TOP | WS_TABSTOP,6,66,107,10
    CONTROL         "Auto Reduction Voice",IDC_CHECKBOX_REDUCE_VOICE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,76,75,8
    GROUPBOX        "Default MIDI Type",IDC_GROUPBOX_DEFAULT_MIDI_TYPE,6,87,
                    87,27
    CONTROL         "GM",IDC_RADIOBUTTON_GM,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,14,99,22,8
    CONTROL         "GS",IDC_RADIOBUTTON_GS,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,39,99,20,8
    CONTROL         "XG",IDC_RADIOBUTTON_XG,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,63,99,20,8
    GROUPBOX        "Others",IDC_STATIC,121,3,100,114
    LTEXT           "Resample Cache",IDC_STATIC,127,15,50,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CACHE_SIZE,179,13,36,12
    LTEXT           "Default tonebank",IDC_STATIC,127,28,50,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_DEFAULT_TONEBANK,183,26,24,12
    CONTROL         "All tonebank",IDC_CHECKBOX_SPECIAL_TONEBANK,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,127,42,54,8
    EDITTEXT        IDC_EDIT_SPECIAL_TONEBANK,183,40,24,12
    LTEXT           "Control Ratio",IDC_STATIC,127,56,40,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CONTROL_RATIO,183,54,24,12
    CONTROL         "Trace Playing",IDC_CHECKBOX_CTL_TRACE_PLAYING,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,127,68,53,8
    LTEXT           "Console Verbosity",IDC_STATIC,127,79,55,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CTL_VEBOSITY,185,77,20,12
    LTEXT           "Drum Power(%)",IDC_STATIC,127,93,47,10,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_DRUM_POWER,185,92,21,12
    CONTROL         "Amplitude Compensation",IDC_CHECKBOX_AMP_COMPENSATION,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,127,107,86,6
    GROUPBOX        "Musical grammar",IDC_STATIC,1,118,198,66
    CONTROL         "Pure intonation(-Zpure)",IDC_CHECKBOX_PURE_INTONATION,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,6,126,90,8
    CONTROL         "Initial tonality",IDC_CHECKBOX_INIT_KEYSIG,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,11,138,51,8
    COMBOBOX        IDC_COMBO_INIT_KEYSIG,64,137,80,53,CBS_DROPDOWNLIST | 
                    WS_VSCROLL | WS_TABSTOP
    CONTROL         "Minor key",IDC_CHECKBOX_INIT_MI,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,148,138,44,8
    LTEXT           "Key shift(-Kn)",IDC_STATIC,6,153,45,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_KEY_ADJUST,53,152,21,11
    CONTROL         "Fix key signature(-Hn)",IDC_CHECKBOX_FORCE_KEYSIG,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,6,169,76,8
    COMBOBOX        IDC_COMBO_FORCE_KEYSIG,84,168,80,53,CBS_DROPDOWNLIST | 
                    WS_VSCROLL | WS_TABSTOP
END

IDD_PREF_TIMIDITY3_EN DIALOG DISCARDABLE  0, 0, 210, 190
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION | WS_TABSTOP
FONT 9, "Times New Roman"
BEGIN
    GROUPBOX        "Output Encoding",IDC_STATIC,0,0,203,140,WS_GROUP | 
                    WS_TABSTOP
    GROUPBOX        "Precision",IDC_STATIC,4,7,192,30,WS_GROUP | WS_TABSTOP
    CONTROL         "uLaw",IDC_CHECKBOX_ULAW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,65,15,31,8
    CONTROL         "ALaw",IDC_CHECKBOX_ALAW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,65,25,31,8
    CONTROL         "Signed",IDC_CHECKBOX_SIGNED,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,101,15,37,8
    CONTROL         "Unsigned",IDC_CHECKBOX_UNSIGNED,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,101,25,37,8
    CONTROL         "Byteswap",IDC_CHECKBOX_BYTESWAP,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,143,15,43,8
    CONTROL         "Linear PCM",IDC_CHECKBOX_LINEAR,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,143,25,45,8
    GROUPBOX        "Channels",IDC_STATIC,4,41,45,33,WS_GROUP | WS_TABSTOP
    CONTROL         "Stereo",IDC_RADIO_STEREO,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,8,51,36,10
    CONTROL         "Mono",IDC_RADIO_MONO,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,8,63,35,8
    GROUPBOX        "Output",IDC_GROUPBOX_OUTPUT,4,78,193,41,WS_GROUP | 
                    WS_TABSTOP
    COMBOBOX        IDC_COMBO_OUTPUT,8,87,151,64,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    PUSHBUTTON      "Option",IDC_BUTTON_OUTPUT_OPTIONS,160,86,34,14
    COMBOBOX        IDC_COMBO_OUTPUT_MODE,8,103,186,72,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    PUSHBUTTON      "Output File",IDC_BUTTON_OUTPUT_FILE,4,121,42,14,
                    BS_CENTER | WS_GROUP
    EDITTEXT        IDC_EDIT_OUTPUT_FILE,47,121,130,15,ES_AUTOHSCROLL | 
                    ES_OEMCONVERT | WS_GROUP
    PUSHBUTTON      "DEL",IDC_BUTTON_OUTPUT_FILE_DEL,178,121,19,14,BS_CENTER
    COMBOBOX        IDC_COMBO_BANDWIDTH,8,19,48,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_SAMPLE_RATE,58,54,36,64,CBS_DROPDOWN | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Sample rate",IDC_STATIC,54,41,61,33,WS_GROUP | 
                    WS_TABSTOP
    LTEXT           "Hz",IDC_STATIC,98,56,10,8
END

IDD_PREF_TIMIDITY4_EN DIALOG DISCARDABLE  0, 0, 211, 206
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "Times New Roman"
BEGIN
    CONTROL         "Ch01",IDC_CHECKBOX_CH01,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,16,28,8
    CONTROL         "Ch02",IDC_CHECKBOX_CH02,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,24,28,8
    CONTROL         "Ch03",IDC_CHECKBOX_CH03,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,32,28,8
    CONTROL         "Ch04",IDC_CHECKBOX_CH04,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,40,28,8
    CONTROL         "Ch05",IDC_CHECKBOX_CH05,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,48,28,8
    CONTROL         "Ch06",IDC_CHECKBOX_CH06,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,56,28,8
    CONTROL         "Ch07",IDC_CHECKBOX_CH07,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,64,28,8
    CONTROL         "Ch08",IDC_CHECKBOX_CH08,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,72,28,8
    CONTROL         "Ch09",IDC_CHECKBOX_CH09,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,16,28,8
    CONTROL         "Ch10",IDC_CHECKBOX_CH10,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,24,28,8
    CONTROL         "Ch11",IDC_CHECKBOX_CH11,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,32,28,8
    CONTROL         "Ch12",IDC_CHECKBOX_CH12,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,40,28,8
    CONTROL         "Ch13",IDC_CHECKBOX_CH13,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,48,28,8
    CONTROL         "Ch14",IDC_CHECKBOX_CH14,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,56,28,8
    CONTROL         "Ch15",IDC_CHECKBOX_CH15,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,64,28,8
    CONTROL         "Ch16",IDC_CHECKBOX_CH16,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,72,28,8
    CONTROL         "Ch17",IDC_CHECKBOX_CH17,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,16,28,8
    CONTROL         "Ch18",IDC_CHECKBOX_CH18,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,24,28,8
    CONTROL         "Ch19",IDC_CHECKBOX_CH19,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,32,28,8
    CONTROL         "Ch20",IDC_CHECKBOX_CH20,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,40,28,8
    CONTROL         "Ch21",IDC_CHECKBOX_CH21,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,48,28,8
    CONTROL         "Ch22",IDC_CHECKBOX_CH22,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,56,28,8
    CONTROL         "Ch23",IDC_CHECKBOX_CH23,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,64,28,8
    CONTROL         "Ch24",IDC_CHECKBOX_CH24,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,72,28,8
    CONTROL         "Ch25",IDC_CHECKBOX_CH25,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,16,28,8
    CONTROL         "Ch26",IDC_CHECKBOX_CH26,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,24,28,8
    CONTROL         "Ch27",IDC_CHECKBOX_CH27,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,32,28,8
    CONTROL         "Ch28",IDC_CHECKBOX_CH28,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,40,28,8
    CONTROL         "Ch29",IDC_CHECKBOX_CH29,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,48,28,8
    CONTROL         "Ch30",IDC_CHECKBOX_CH30,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,56,28,8
    CONTROL         "Ch31",IDC_CHECKBOX_CH31,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,64,28,8
    CONTROL         "Ch32",IDC_CHECKBOX_CH32,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,72,28,8
    CONTROL         "Default Drum Channel",IDC_CHECKBOX_DRUM_CHANNEL,"Button",
                    BS_AUTOCHECKBOX | BS_PUSHLIKE | WS_TABSTOP,9,87,95,13
    CONTROL         "Ignore Auto Detect Channel",
                    IDC_CHECKBOX_DRUM_CHANNEL_MASK,"Button",BS_AUTOCHECKBOX | 
                    BS_PUSHLIKE | WS_TABSTOP,9,105,95,14
    CONTROL         "Mute Channel",IDC_CHECKBOX_QUIET_CHANNEL,"Button",
                    BS_AUTOCHECKBOX | BS_PUSHLIKE | WS_TABSTOP,9,124,95,13
    PUSHBUTTON      "Reverse",IDC_BUTTON_REVERSE,141,18,31,61
    GROUPBOX        "Channel",IDC_STATIC,1,3,179,138
    GROUPBOX        "Mute temperament type",IDC_STATIC,1,144,195,39
    CONTROL         "Equal",IDC_CHECKBOX_TEMPER_EQUAL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,5,155,37,8
    CONTROL         "Pythagoras",IDC_CHECKBOX_TEMPER_PYTHA,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,55,155,46,8
    CONTROL         "Mean-tone",IDC_CHECKBOX_TEMPER_MEANTONE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,105,155,45,8
    CONTROL         "Pure int.",IDC_CHECKBOX_TEMPER_PUREINT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,155,155,35,8
    CONTROL         "User #0",IDC_CHECKBOX_TEMPER_USER0,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,5,168,45,8
    CONTROL         "User #1",IDC_CHECKBOX_TEMPER_USER1,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,55,168,45,8
    CONTROL         "User #2",IDC_CHECKBOX_TEMPER_USER2,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,105,168,45,8
    CONTROL         "User #3",IDC_CHECKBOX_TEMPER_USER3,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,155,168,36,8
END

IDD_DIALOG_VORBIS_EN DIALOG DISCARDABLE  0, 0, 197, 93
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Ogg Vorbis Configurations"
FONT 9, "Times New Roman"
BEGIN
    CONTROL         "Default",IDC_CHECK_DEFAULT,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,4,0,56,10
    COMBOBOX        IDC_COMBO_MODE,25,13,123,56,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Title",IDC_STATIC,4,49,14,8
    EDITTEXT        IDC_EDIT1,41,48,152,14,ES_AUTOHSCROLL | WS_DISABLED
    LTEXT           "Artist",IDC_STATIC,4,63,18,8
    EDITTEXT        IDC_EDIT2,41,76,152,14,ES_AUTOHSCROLL | WS_DISABLED
    LTEXT           "Arbum",IDC_STATIC,4,77,27,8
    EDITTEXT        IDC_EDIT3,41,62,152,14,ES_AUTOHSCROLL | WS_DISABLED
    CONTROL         "Store follows information",IDC_CHECK_USE_TAG,"Button",
                    BS_AUTOCHECKBOX | WS_DISABLED | WS_TABSTOP,4,32,73,10
    LTEXT           "Mode",IDC_STATIC,4,16,18,8
    PUSHBUTTON      "OK",IDOK,152,2,40,12
    PUSHBUTTON      "Cancel",IDCANCEL,152,30,40,12
    PUSHBUTTON      "Apply",IDC_BUTTON_APPLY,152,14,40,11
END

IDD_DIALOG_GOGO_EN DIALOG DISCARDABLE  0, 0, 346, 196
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "MP3 (gogo) Configuration"
FONT 8, "Times New Roman"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,173,179,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,285,179,50,14
    COMBOBOX        IDC_COMBO_OUTPUT_FORMAT,4,36,72,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Default",IDC_CHECK_DEFAULT,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,4,0,80,10
    CONTROL         "Command line option",IDC_CHECK_COMMANDLINE_OPTS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,4,12,86,10
    EDITTEXT        IDC_EDIT_COMMANDLINE_OPTION,91,10,232,14,ES_AUTOHSCROLL
    COMBOBOX        IDC_COMBO_MPEG1_AUDIO_BITRATE,6,72,76,76,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_MPEG2_AUDIO_BITRATE,87,72,76,76,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_VBR_BITRATE_LOW,6,119,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "VBR - Maximum",IDC_STATIC,86,110,55,8
    COMBOBOX        IDC_COMBO_VBR_BITRATE_HIGH,86,119,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "16KHz Lowpath filter",IDC_CHECK_16KHZ_LOW_PASS_FILTER,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,176,57,89,8
    CONTROL         "Enhanced Lowpath Filter",
                    IDC_CHECK_ENHANCED_LOW_PASS_FILTER,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,176,68,104,8
    EDITTEXT        IDC_EDIT_LPF_PARA1,200,77,28,12
    EDITTEXT        IDC_EDIT_LPF_PARA2,251,77,28,12
    LTEXT           "para1",IDC_STATIC,183,79,16,8
    LTEXT           "para2",IDC_STATIC,233,79,16,8
    COMBOBOX        IDC_COMBO_ENCODE_MODE,172,36,84,60,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Use Psy",IDC_CHECK_USEPSY,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,90,42,10
    CONTROL         "MMX",IDC_CHECK_CPUMMX,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,193,151,29,10
    CONTROL         "SSE",IDC_CHECK_CPUSSE,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,193,163,27,10
    CONTROL         "3DNOW",IDC_CHECK_CPU3DNOW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,228,152,39,10
    CONTROL         "E3DNOW",IDC_CHECK_CPUE3DNOW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,227,163,45,10
    COMBOBOX        IDC_COMBO_EMPHASIS_TYPE,268,36,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_VBR,37,92,124,72,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    CONTROL         "VBR",IDC_CHECK_VBR,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,94,29,10
    CONTROL         "Output Frequency",IDC_CHECK_OUTFREQ,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,88,28,75,8
    EDITTEXT        IDC_EDIT_OUTFREQ,88,36,72,12
    CONTROL         "J-Stereo Stereo/MS-Stereo Control",
                    IDC_CHECK_MSTHRESHOLD,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,103,167,8
    EDITTEXT        IDC_EDIT_MSTHRESHOLD_THRESHOLD,200,112,28,12
    EDITTEXT        IDC_EDIT_MSTHRESHOLD_MSPOWER,252,112,28,12
    LTEXT           "para1",IDC_STATIC,183,114,16,8
    LTEXT           "para2",IDC_STATIC,233,114,16,8
    CONTROL         "Verify",IDC_CHECK_VERIFY,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,127,34,10
    PUSHBUTTON      "Apply",IDC_BUTTON_APPLY,229,179,50,14
    CONTROL         "Use follow options",IDC_CHECK_USE_CPU_OPTS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,176,140,68,8
    CONTROL         "MPEG1Audio",IDC_CHECK_MPEG1AUDIOBITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,64,80,8
    CONTROL         "MPEG2Audio",IDC_CHECK_MPEG2AUDIOBITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,87,64,80,8
    CONTROL         "VBR - Minimum",IDC_CHECK_VBR_BITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,110,67,8
    CONTROL         "Output format",IDC_CHECK_OUTPUT_FORMAT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,4,28,64,8
    CONTROL         "Encode Mode",IDC_CHECK_ENCODE_MODE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,172,28,63,8
    CONTROL         "Emphasis type",IDC_CHECK_EMPHASIS_TYPE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,268,28,62,8
    GROUPBOX        "Bitrate",IDC_STATIC,2,55,167,86
    CONTROL         "CMOV",IDC_CHECK_CPUCMOV,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,280,140,39,10
    CONTROL         "EMMX",IDC_CHECK_CPUEMMX,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,280,152,39,10
    CONTROL         "SSE2",IDC_CHECK_CPUSSE2,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,280,164,39,10
END

IDD_DIALOG_ONE_LINE_EN DIALOGEX 0, 0, 187, 63
STYLE DS_MODALFRAME | DS_3DLOOK | WS_POPUP | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_CONTROLPARENT
FONT 10, "Times New Roman", 0, 0, 0x1
BEGIN
    EDITTEXT        IDC_EDIT_ONE_LINE,4,16,180,14,ES_AUTOHSCROLL | WS_GROUP
    DEFPUSHBUTTON   "",IDC_BUTTON_1,32,48,50,14
    DEFPUSHBUTTON   "",IDC_BUTTON_2,84,48,50,14
    DEFPUSHBUTTON   "",IDC_BUTTON_3,136,48,50,14
    LTEXT           "",IDC_STATIC_TAIL,4,32,180,12
    LTEXT           "",IDC_STATIC_HEAD,4,0,180,12
END

IDD_PREF_SYN1_EN DIALOGEX 0, 0, 207, 166
STYLE DS_CONTROL | WS_CHILD | WS_CAPTION
FONT 9, "Times New Roman", 0, 0, 0x1
BEGIN
    LTEXT           "Port 0",IDC_STATIC,6,27,24,10
    LTEXT           "Port 1",IDC_STATIC,6,42,24,10
    LTEXT           "Port 2",IDC_STATIC,6,57,24,10
    LTEXT           "Port 3",IDC_STATIC,6,72,24,10
    GROUPBOX        "MIDI IN Device",IDC_STATIC,3,0,168,90
    COMBOBOX        IDC_COMBO_IDPORT0,33,27,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT1,33,42,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT2,33,57,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT3,33,72,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Synth thread priority",IDC_STATIC,7,123,62,10
    COMBOBOX        IDC_COMBO_SYN_THREAD_PRIORITY,70,121,102,63,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Port max",IDC_STATIC,6,12,36,10
    COMBOBOX        IDC_COMBO_PORT_NUM,45,12,39,81,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Compute max time msec(default 500ms)",IDC_STATIC,7,137,
                    114,9
    EDITTEXT        IDC_EDIT_SYN_SH_TIME,127,137,54,12,ES_AUTOHSCROLL
    LTEXT           "Process priority",IDC_STATIC,7,109,62,10
    COMBOBOX        IDC_COMBO_PROCESS_PRIORITY,70,107,102,63,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Automatically start synthesize",IDC_CHECK_SYN_AUTOSTART,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,7,96,115,8
END

IDD_DIALOG_PREF_EN DIALOG DISCARDABLE  0, 0, 235, 229
STYLE DS_MODALFRAME | WS_MAXIMIZEBOX | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "TiMidity++ Win32GUI Preference"
FONT 9, "Times New Roman"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,57,213,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,117,213,50,14
    PUSHBUTTON      "Apply",IDC_BUTTON_APPLY,174,213,50,14
    CONTROL         "Tab1",IDC_TAB_MAIN,"SysTabControl32",0x0,0,0,234,210
END


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDM_MENU_MAIN_EN MENU DISCARDABLE 
BEGIN
    POPUP "File(&F)"
    BEGIN
        MENUITEM "Open File(&F)",               IDM_MFOPENFILE
        MENUITEM "Open Directory(&D)",          IDM_MFOPENDIR
        MENUITEM SEPARATOR
        MENUITEM "Load Playlist(&P)",           IDM_MFLOADPLAYLIST
        MENUITEM "Save Playlist as(&S)",        IDM_MFSAVEPLAYLISTAS
        MENUITEM SEPARATOR
        MENUITEM "Exit(&X)",                    IDM_MFEXIT
    END
    POPUP "Config(&C)"
    BEGIN
        MENUITEM "Preference(&P)",              IDM_SETTING
        MENUITEM SEPARATOR
        MENUITEM "Load ini file(&L)",           IDM_MCLOADINIFILE
        MENUITEM "Save ini file(&S)",           IDM_MCSAVEINIFILE
        MENUITEM SEPARATOR
        MENUITEM "Reload cfg file(&F)",         IDM_FORCE_RELOAD
    END
    POPUP "Window(&W)"
    BEGIN
        MENUITEM "Play List(&L)",               IDM_MWPLAYLIST
        MENUITEM "Tracer(&T)",                  IDM_MWTRACER
        MENUITEM "Document(&D)",                IDM_MWDOCUMENT
        MENUITEM "Wrd tracer(&W)",              IDM_MWWRDTRACER
        MENUITEM "Console(&C)",                 IDM_MWCONSOLE
    END
    POPUP "Help(&H)"
    BEGIN
        MENUITEM "Online Help(&O)",             IDM_MHONLINEHELP
        MENUITEM "Bug Tracking System(&B)",     IDM_MHBTS
        MENUITEM SEPARATOR
        MENUITEM "TiMidity++(&T)",              IDM_MHTIMIDITY
        MENUITEM "Version(&V)",                 IDM_MHVERSION
        MENUITEM SEPARATOR
        MENUITEM "Supplement(&S)",              IDM_MHSUPPLEMENT
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_BITMAP_MAIN_BUTTON  BITMAP  MOVEABLE PURE   "w32g_btn.bmp"
IDB_BITMAP_SUBWND_BUTTON BITMAP  MOVEABLE PURE   "w32g_subbtn.bmp"
IDB_BITMAP_SLEEP        BITMAP  MOVEABLE PURE   "w32g_sleep.bmp"
IDB_BITMAP_TRACER       BITMAP  MOVEABLE PURE   "w32g_tracer.bmp"

/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO MOVEABLE PURE 
BEGIN
    IDD_DIALOG_CONSOLE_EN, DIALOG
    BEGIN
        RIGHTMARGIN, 247
    END

    IDD_DIALOG_TRACER, DIALOG
    BEGIN
        BOTTOMMARGIN, 220
    END

    IDD_DIALOG_SIMPLE_LIST_EN, DIALOG
    BEGIN
        RIGHTMARGIN, 241
        BOTTOMMARGIN, 179
    END

    IDD_PREF_TIMIDITY1_EN, DIALOG
    BEGIN
        RIGHTMARGIN, 218
        BOTTOMMARGIN, 164
    END

    IDD_DIALOG_ONE_LINE_EN, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 180
        TOPMARGIN, 7
        BOTTOMMARGIN, 56
    END

    IDD_PREF_SYN1_EN, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 200
        TOPMARGIN, 7
        BOTTOMMARGIN, 159
    END

    IDD_DIALOG_PREF_EN, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 228
        TOPMARGIN, 7
        BOTTOMMARGIN, 222
    END
END
#endif    // APSTUDIO_INVOKED

#endif    // ﾆｭｰﾄﾗﾙ resources
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// 日本語 resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_JPN)
#ifdef _WIN32
LANGUAGE LANG_JAPANESE, SUBLANG_DEFAULT
#pragma code_page(932)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_DIALOG_CONSOLE DIALOGEX 0, 0, 243, 184
STYLE DS_MODALFRAME | DS_3DLOOK | WS_CAPTION | WS_SYSMENU
CAPTION "コンソール (TiMidity++ Win32GUI)"
FONT 9, "ＭＳ Ｐゴシック", 0, 0, 0x1
BEGIN
    PUSHBUTTON      "クリア",IDCLEAR,140,167,96,16,BS_CENTER
    EDITTEXT        IDC_EDIT_VERBOSITY,44,167,16,16,ES_OEMCONVERT
    CONTROL         "有効",IDC_CHECKBOX_VALID,"Button",BS_AUTOCHECKBOX | 
                    BS_LEFTTEXT | WS_TABSTOP,97,167,27,16
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,4,167,80,16
    EDITTEXT        IDC_EDIT,0,0,242,160,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | ES_WANTRETURN | 
                    WS_VSCROLL | WS_HSCROLL,WS_EX_STATICEDGE
    PUSHBUTTON      "レベル",IDC_BUTTON_VERBOSITY,4,167,40,16,BS_CENTER,
                    WS_EX_CLIENTEDGE
    PUSHBUTTON      "<<",IDC_BUTTON_DEC,60,167,12,16,BS_CENTER
    PUSHBUTTON      ">>",IDC_BUTTON_INC,72,167,12,16,BS_CENTER
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,93,167,38,16
END

IDD_DIALOG_DOC DIALOGEX 0, 0, 302, 257
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME
EXSTYLE WS_EX_ACCEPTFILES
CAPTION "ドキュメント (TiMidity++ Win32GUI)"
FONT 9, "ＭＳ Ｐゴシック", 0, 0, 0x1
BEGIN
    EDITTEXT        IDC_EDIT_FILENAME,100,240,150,16,ES_CENTER | ES_READONLY
    PUSHBUTTON      "<<",IDC_BUTTON_PREV,255,240,20,16,BS_CENTER
    PUSHBUTTON      ">>",IDC_BUTTON_NEXT,275,240,20,16,BS_CENTER
    EDITTEXT        IDC_EDIT,0,0,295,220,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | ES_WANTRETURN | 
                    WS_VSCROLL | WS_HSCROLL
    EDITTEXT        IDC_EDIT_INFO,0,240,95,16,ES_READONLY
END

IDD_DIALOG_WRD DIALOGEX 0, 0, 366, 215
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_ACCEPTFILES
CAPTION "Wrd Tracer Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman", 400, 0, 0x1
BEGIN
END

IDD_DIALOG_SOUNDSPEC DIALOG DISCARDABLE  0, 0, 240, 180
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU
CAPTION "Sound Spec Window (TiMidity Win32GUI)"
FONT 9, "Times New Roman"
BEGIN
    PUSHBUTTON      "C L E A R",IDCLEAR,136,164,96,16,BS_CENTER
    CONTROL         "VALID",IDC_CHECKBOX_DEBUG_WND_VALID,"Button",
                    BS_AUTOCHECKBOX | BS_LEFTTEXT | WS_TABSTOP,20,164,36,16
    CONTROL         "",-1,"Static",SS_ETCHEDFRAME,16,164,44,16
END

IDD_DIALOG_SIMPLE_LIST DIALOGEX 0, 0, 242, 189
STYLE DS_3DLOOK | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME
EXSTYLE WS_EX_DLGMODALFRAME | WS_EX_ACCEPTFILES | WS_EX_CONTROLPARENT
CAPTION "プレイリスト (TiMidity++ Win32GUI)"
FONT 9, "ＭＳ Ｐゴシック", 0, 0, 0x1
BEGIN
    LISTBOX         IDC_LISTBOX_PLAYLIST,0,0,241,172,LBS_HASSTRINGS | 
                    LBS_WANTKEYBOARDINPUT | LBS_DISABLENOSCROLL | WS_VSCROLL | 
                    WS_HSCROLL | WS_GROUP | WS_TABSTOP
    PUSHBUTTON      "ドキュメント",IDC_BUTTON_DOC,54,175,36,12,BS_CENTER
    PUSHBUTTON      "重複削除",IDC_BUTTON_UNIQ,117,175,31,12,BS_CENTER
    PUSHBUTTON      "掃除",IDC_BUTTON_REFINE,150,175,22,12,BS_CENTER
    PUSHBUTTON      "クリア",IDC_BUTTON_CLEAR,200,175,42,12,BS_CENTER | 
                    WS_GROUP
    EDITTEXT        IDC_EDIT_NUM,5,175,45,12,ES_READONLY
END

IDD_PREF_PLAYER DIALOGEX 0, 0, 211, 215
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    PUSHBUTTON      "設定ファイル",IDC_BUTTON_CONFIG_FILE,3,11,52,14,
                    BS_CENTER
    EDITTEXT        IDC_EDIT_CONFIG_FILE,56,11,127,15,ES_AUTOHSCROLL | 
                    ES_OEMCONVERT
    GROUPBOX        "プレイヤー",IDC_GROUPBOX_PLAYERMODE,2,52,106,61,
                    WS_GROUP
    CONTROL         "一曲演奏して停止",IDC_CHECKBOX_NOT_CONTINUE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,7,64,64,8
    CONTROL         "ドラッグ&&ドロップで再生しない",
                    IDC_CHECKBOX_NOT_DRAG_START,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,7,76,97,8
    CONTROL         "リピートしない",IDC_CHECKBOX_NOT_LOOPING,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,7,88,51,8
    CONTROL         "オートスタート",IDC_CHECKBOX_AUTOSTART,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,7,100,56,8
    GROUPBOX        "プレイリスト",IDC_STATIC,112,52,88,85
    CONTROL         "全曲演奏後終了",IDC_CHECKBOX_AUTOQUIT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,64,61,8
    CONTROL         "重複登録を防止",IDC_CHECKBOX_AUTOUNIQ,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,76,59,8
    CONTROL         "演奏ファイルのみ登録",IDC_CHECKBOX_AUTOREFINE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,88,75,8
    CONTROL         "ランダム再生",IDC_CHECKBOX_RANDOM,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,100,50,8
    CONTROL         "自動読込",IDC_CHECK_AUTOLOAD_PLAYLIST,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,111,50,8
    CONTROL         "自動保存",IDC_CHECK_AUTOSAVE_PLAYLIST,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,124,50,8
    GROUPBOX        "サブウインドウ",IDC_STATIC,4,116,104,25
    LTEXT           "最大数",IDC_STATIC,8,128,24,8
    EDITTEXT        IDC_EDIT_SUBWINDOW_MAX,32,125,24,12,ES_CENTER,
                    WS_EX_RIGHT
    GROUPBOX        "ドキュメント",IDC_STATIC,2,142,75,38
    CONTROL         "自動表示しない",IDC_CHECK_DOCWNDINDEPENDENT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,153,56,8
    CONTROL         "ポップアップ",IDC_CHECK_DOCWNDAUTOPOPUP,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,165,59,10
    GROUPBOX        "ウインドウ",IDC_STATIC,3,183,72,28
    CONTROL         "位置・サイズ保存",IDC_CHECK_POS_SIZE_SAVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,8,196,64,10
    GROUPBOX        "フォルダ",IDC_STATIC,83,142,70,23
    CONTROL         "再帰検索",IDC_CHECK_SEACHDIRRECURSIVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,88,152,41,10
    GROUPBOX        "設定",IDC_STATIC,83,170,70,25
    CONTROL         "設定の自動保存",IDC_CHECK_INIFILE_AUTOSAVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,88,181,59,10
    GROUPBOX        "Language",IDC_GROUPBOX_LANGUAGE,160,142,43,38,WS_GROUP
    CONTROL         "日本語",IDC_RADIOBUTTON_JAPANESE,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,163,154,30,8
    CONTROL         "English",IDC_RADIOBUTTON_ENGLISH,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,163,167,36,8
    PUSHBUTTON      "設定ファイルの強制再読込",IDC_BUTTON_CFG_RELOAD,3,31,96,
                    15,BS_MULTILINE
    PUSHBUTTON      "編集",IDC_BUTTON_CFG_EDIT,184,11,25,14,BS_CENTER
END

IDD_PREF_TIMIDITY1 DIALOG DISCARDABLE  0, 0, 211, 207
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    CONTROL         "モジュレーションホイール",IDC_CHECKBOX_MODWHEEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,75,85,10
    CONTROL         "ポルタメント",IDC_CHECKBOX_PORTAMENTO,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,88,50,10
    CONTROL         "NRPN ビブラート",IDC_CHECKBOX_NRPNVIB,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,127,65,10
    CONTROL         "汎用コントローラ",IDC_CHECKBOX_CHPRESS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,101,80,10
    CONTROL         "テキストメタイベント",IDC_CHECKBOX_TRACETEXT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,101,59,73,10
    CONTROL         "同音連打",IDC_CHECKBOX_OVOICE,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,101,45,43,10
    CONTROL         "アタック・タイム",IDC_CHECKBOX_TVAA,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,140,59,10
    CONTROL         "ディケイ・タイム",IDC_CHECKBOX_TVAD,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,153,61,10
    CONTROL         "リリース・タイム",IDC_CHECKBOX_TVAR,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,166,61,10
    CONTROL         "EQ",IDC_CHECKBOX_EQ,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,179,25,10
    CONTROL         "インサーション・エフェクト",IDC_CHECKBOX_IEFFECT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,192,84,10
    CONTROL         "コーラス・レベル",IDC_CHECKBOX_CHORUS_LEVEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,101,17,63,10
    EDITTEXT        IDC_EDIT_CHORUS,167,16,24,10,ES_NUMBER
    CONTROL         "リバーブ・レベル",IDC_CHECKBOX_REVERB_LEVEL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,101,31,63,10
    EDITTEXT        IDC_EDIT_REVERB,167,30,24,10,ES_NUMBER
    CONTROL         "L&&R ディレイ",IDC_CHECKBOX_DELAY,"Button",
                    BS_AUTOCHECKBOX | BS_LEFT | BS_TOP | WS_TABSTOP,101,112,
                    57,10
    CONTROL         "左",IDC_RADIOBUTTON_DELAY_LEFT,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,109,127,23,10
    CONTROL         "右",IDC_RADIOBUTTON_DELAY_RIGHT,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,135,127,23,10
    CONTROL         "回転",IDC_RADIOBUTTON_DELAY_CENTER,"Button",
                    BS_AUTORADIOBUTTON | WS_TABSTOP,161,127,30,10
    LTEXT           "ディレイタイム(ms)",IDC_STATIC,113,141,55,8,NOT 
                    WS_GROUP | WS_TABSTOP
    EDITTEXT        IDC_EDIT_DELAY,171,139,32,10,ES_NUMBER
    LTEXT           "疑似リバーブ(0..1000)",IDC_STATIC,101,155,64,8,NOT 
                    WS_GROUP | WS_TABSTOP
    EDITTEXT        IDC_EDIT_MODIFY_RELEASE,167,153,28,10
    LTEXT           "ノイズシェイピング (0..6)",IDC_STATIC,101,87,70,8,NOT 
                    WS_GROUP | WS_TABSTOP
    EDITTEXT        IDC_EDIT_NOISESHARPING,176,85,16,10,ES_NUMBER
    CONTROL         "NRPN ドラム・エフェクト",IDC_CHECKBOX_DRUM_EFFECT,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,6,114,81,10
    GROUPBOX        "GM/GS/XG",IDC_STATIC,1,3,94,204
    GROUPBOX        "特殊",IDC_STATIC,97,3,113,163
    CONTROL         "モジュレーション・エンベロープ",IDC_CHECKBOX_MOD_ENV,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,101,73,100,10
    COMBOBOX        IDC_COMBO_REVERB,6,15,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_CHORUS,6,30,84,54,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_DELAY,6,45,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_LPF,6,60,84,78,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_MODULE,101,188,103,78,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "音源モード",IDC_STATIC,97,176,113,31
    CONTROL         "サラウンド",IDC_CHECKBOX_PAN_DELAY,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,101,97,41,13
END

IDD_PREF_TIMIDITY2 DIALOG DISCARDABLE  0, 0, 211, 217
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    GROUPBOX        "再生",IDC_STATIC,1,3,107,133
    LTEXT           "ボイス",IDC_STATIC,6,19,20,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_VOICES,31,17,24,12,ES_NUMBER
    LTEXT           "音量(%)",IDC_STATIC,6,33,23,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_AMPLIFICATION,31,31,24,12
    CONTROL         "アンチエイリアス",IDC_CHECKBOX_ANTIALIAS,"Button",
                    BS_AUTOCHECKBOX | BS_LEFT | BS_TOP | WS_TABSTOP,6,49,64,
                    10
    CONTROL         "演奏中に音色読込",IDC_CHECKBOX_LOADINST_PLAYING,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,62,67,10
    CONTROL         "演奏毎に音色破棄",IDC_CHECKBOX_FREE_INST,"Button",
                    BS_AUTOCHECKBOX | BS_LEFT | BS_TOP | WS_TABSTOP,6,75,66,
                    10
    CONTROL         "高負荷時にボイス削減",IDC_CHECKBOX_REDUCE_VOICE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,89,82,8
    GROUPBOX        "デフォルト規格",IDC_GROUPBOX_DEFAULT_MIDI_TYPE,6,105,87,
                    27
    CONTROL         "GM",IDC_RADIOBUTTON_GM,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,14,117,22,8
    CONTROL         "GS",IDC_RADIOBUTTON_GS,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,39,117,20,8
    CONTROL         "XG",IDC_RADIOBUTTON_XG,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,63,117,20,8
    GROUPBOX        "高度な設定",IDC_STATIC,111,3,100,133
    LTEXT           "キャッシュ",IDC_STATIC,117,19,31,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CACHE_SIZE,149,17,39,12
    LTEXT           "デフォルトバンク",IDC_STATIC,117,34,49,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_DEFAULT_TONEBANK,169,32,24,12
    CONTROL         "バンク固定",IDC_CHECKBOX_SPECIAL_TONEBANK,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,49,43,8
    EDITTEXT        IDC_EDIT_SPECIAL_TONEBANK,164,47,24,12
    LTEXT           "制御レシオ",IDC_STATIC,117,65,33,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CONTROL_RATIO,153,63,24,12
    CONTROL         "トレースモード",IDC_CHECKBOX_CTL_TRACE_PLAYING,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,80,54,8
    LTEXT           "コンソールレベル",IDC_STATIC,117,93,51,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_CTL_VEBOSITY,170,91,20,12
    LTEXT           "ドラム音量補正(%)",IDC_STATIC,117,108,54,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_DRUM_POWER,175,106,21,12
    CONTROL         "音量自動補正",IDC_CHECKBOX_AMP_COMPENSATION,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,117,123,54,8
    GROUPBOX        "楽典",IDC_STATIC,1,137,209,80
    CONTROL         "純正律で演奏する(-Zpure)",IDC_CHECKBOX_PURE_INTONATION,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,6,148,90,10
    CONTROL         "初期調性",IDC_CHECKBOX_INIT_KEYSIG,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,11,164,37,8
    COMBOBOX        IDC_COMBO_INIT_KEYSIG,51,161,80,53,CBS_DROPDOWNLIST | 
                    WS_VSCROLL | WS_TABSTOP
    CONTROL         "短調",IDC_CHECKBOX_INIT_MI,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,137,164,28,10
    LTEXT           "音程シフト(-Kn)",IDC_STATIC,6,184,47,8,NOT WS_GROUP
    EDITTEXT        IDC_EDIT_KEY_ADJUST,56,181,21,12
    CONTROL         "調号の数を固定して演奏する(-Hn)",
                    IDC_CHECKBOX_FORCE_KEYSIG,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,203,113,8
    COMBOBOX        IDC_COMBO_FORCE_KEYSIG,122,200,80,53,CBS_DROPDOWNLIST | 
                    WS_VSCROLL | WS_TABSTOP
END

IDD_PREF_TIMIDITY3 DIALOG DISCARDABLE  0, 0, 210, 181
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    GROUPBOX        "出力",IDC_STATIC,1,0,203,173,WS_GROUP | WS_TABSTOP
    GROUPBOX        "出力モード",IDC_STATIC,4,14,193,33,WS_GROUP | 
                    WS_TABSTOP
    CONTROL         "U-Law",IDC_CHECKBOX_ULAW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,62,23,31,8
    CONTROL         "A-Law",IDC_CHECKBOX_ALAW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,62,35,31,8
    CONTROL         "Signed",IDC_CHECKBOX_SIGNED,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,98,23,37,8
    CONTROL         "Unsigned",IDC_CHECKBOX_UNSIGNED,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,98,35,37,8
    CONTROL         "バイトスワップ",IDC_CHECKBOX_BYTESWAP,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,140,23,52,8
    CONTROL         "リニアPCM",IDC_CHECKBOX_LINEAR,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,140,35,45,8
    GROUPBOX        "チャンネル",IDC_STATIC,4,55,45,33,WS_GROUP | WS_TABSTOP
    CONTROL         "ステレオ",IDC_RADIO_STEREO,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,8,65,36,10
    CONTROL         "モノラル",IDC_RADIO_MONO,"Button",BS_AUTORADIOBUTTON | 
                    WS_TABSTOP,8,77,35,8
    LTEXT           "Hz",IDC_STATIC,103,72,8,8,WS_TABSTOP
    GROUPBOX        "出力方法",IDC_GROUPBOX_OUTPUT,4,97,193,49,WS_GROUP | 
                    WS_TABSTOP
    COMBOBOX        IDC_COMBO_OUTPUT,8,111,151,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    PUSHBUTTON      "オプション",IDC_BUTTON_OUTPUT_OPTIONS,160,110,34,14
    COMBOBOX        IDC_COMBO_OUTPUT_MODE,8,129,186,72,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    PUSHBUTTON      "出力ファイル",IDC_BUTTON_OUTPUT_FILE,4,151,42,14,
                    BS_CENTER | WS_GROUP
    EDITTEXT        IDC_EDIT_OUTPUT_FILE,47,151,130,15,ES_AUTOHSCROLL | 
                    ES_OEMCONVERT | WS_GROUP
    PUSHBUTTON      "削除",IDC_BUTTON_OUTPUT_FILE_DEL,178,151,19,14,
                    BS_CENTER
    COMBOBOX        IDC_COMBO_BANDWIDTH,8,27,48,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_SAMPLE_RATE,63,69,36,64,CBS_DROPDOWN | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "サンプルレート",IDC_STATIC,56,55,60,33
END

IDD_PREF_TIMIDITY4 DIALOG DISCARDABLE  0, 0, 211, 195
STYLE DS_SYSMODAL | DS_3DLOOK | DS_CONTROL | WS_CHILD | WS_VISIBLE | 
    WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    CONTROL         "Ch01",IDC_CHECKBOX_CH01,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,16,28,8
    CONTROL         "Ch02",IDC_CHECKBOX_CH02,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,24,28,8
    CONTROL         "Ch03",IDC_CHECKBOX_CH03,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,32,28,8
    CONTROL         "Ch04",IDC_CHECKBOX_CH04,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,40,28,8
    CONTROL         "Ch05",IDC_CHECKBOX_CH05,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,48,28,8
    CONTROL         "Ch06",IDC_CHECKBOX_CH06,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,56,28,8
    CONTROL         "Ch07",IDC_CHECKBOX_CH07,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,64,28,8
    CONTROL         "Ch08",IDC_CHECKBOX_CH08,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,10,72,28,8
    CONTROL         "Ch09",IDC_CHECKBOX_CH09,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,16,28,8
    CONTROL         "Ch10",IDC_CHECKBOX_CH10,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,24,28,8
    CONTROL         "Ch11",IDC_CHECKBOX_CH11,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,32,28,8
    CONTROL         "Ch12",IDC_CHECKBOX_CH12,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,40,28,8
    CONTROL         "Ch13",IDC_CHECKBOX_CH13,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,48,28,8
    CONTROL         "Ch14",IDC_CHECKBOX_CH14,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,56,28,8
    CONTROL         "Ch15",IDC_CHECKBOX_CH15,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,64,28,8
    CONTROL         "Ch16",IDC_CHECKBOX_CH16,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,41,72,28,8
    CONTROL         "Ch17",IDC_CHECKBOX_CH17,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,16,28,8
    CONTROL         "Ch18",IDC_CHECKBOX_CH18,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,24,28,8
    CONTROL         "Ch19",IDC_CHECKBOX_CH19,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,32,28,8
    CONTROL         "Ch20",IDC_CHECKBOX_CH20,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,40,28,8
    CONTROL         "Ch21",IDC_CHECKBOX_CH21,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,48,28,8
    CONTROL         "Ch22",IDC_CHECKBOX_CH22,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,56,28,8
    CONTROL         "Ch23",IDC_CHECKBOX_CH23,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,64,28,8
    CONTROL         "Ch24",IDC_CHECKBOX_CH24,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,73,72,28,8
    CONTROL         "Ch25",IDC_CHECKBOX_CH25,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,16,28,8
    CONTROL         "Ch26",IDC_CHECKBOX_CH26,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,24,28,8
    CONTROL         "Ch27",IDC_CHECKBOX_CH27,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,32,28,8
    CONTROL         "Ch28",IDC_CHECKBOX_CH28,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,40,28,8
    CONTROL         "Ch29",IDC_CHECKBOX_CH29,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,48,28,8
    CONTROL         "Ch30",IDC_CHECKBOX_CH30,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,56,28,8
    CONTROL         "Ch31",IDC_CHECKBOX_CH31,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,64,28,8
    CONTROL         "Ch32",IDC_CHECKBOX_CH32,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,106,72,28,8
    CONTROL         "デフォルトのドラムチャンネル",IDC_CHECKBOX_DRUM_CHANNEL,
                    "Button",BS_AUTOCHECKBOX | BS_PUSHLIKE | WS_TABSTOP,9,87,
                    95,13
    CONTROL         "ドラムパート自動認識を無視するチャンネル",
                    IDC_CHECKBOX_DRUM_CHANNEL_MASK,"Button",BS_AUTOCHECKBOX | 
                    BS_PUSHLIKE | WS_TABSTOP,9,105,137,14
    CONTROL         "ミュートするチャンネル",IDC_CHECKBOX_QUIET_CHANNEL,
                    "Button",BS_AUTOCHECKBOX | BS_PUSHLIKE | WS_TABSTOP,9,
                    124,76,13
    PUSHBUTTON      "全て反転",IDC_BUTTON_REVERSE,141,18,31,61
    GROUPBOX        "チャンネル",IDC_STATIC,1,3,179,138
    GROUPBOX        "ミュートする音律種別",IDC_STATIC,1,143,197,49
    CONTROL         "平均律",IDC_CHECKBOX_TEMPER_EQUAL,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,5,157,37,10
    CONTROL         "ピタゴラス音律",IDC_CHECKBOX_TEMPER_PYTHA,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,48,157,59,10
    CONTROL         "中全音律",IDC_CHECKBOX_TEMPER_MEANTONE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,108,157,43,10
    CONTROL         "純正律",IDC_CHECKBOX_TEMPER_PUREINT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,154,157,37,10
    CONTROL         "ユーザ#0",IDC_CHECKBOX_TEMPER_USER0,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,5,173,43,10
    CONTROL         "ユーザ#1",IDC_CHECKBOX_TEMPER_USER1,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,48,173,43,10
    CONTROL         "ユーザ#2",IDC_CHECKBOX_TEMPER_USER2,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,108,173,43,10
    CONTROL         "ユーザ#3",IDC_CHECKBOX_TEMPER_USER3,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,154,173,43,10
END

IDD_DIALOG_WAVEFORM DIALOGEX 0, 0, 240, 120
STYLE DS_MODALFRAME | DS_3DLOOK | WS_VISIBLE | WS_CAPTION
EXSTYLE WS_EX_DLGMODALFRAME
FONT 9, "Times New Roman", 400, 0, 0x1
BEGIN
END

IDD_DIALOG_SEARCHBOX DIALOG DISCARDABLE  0, 0, 183, 54
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Seach"
FONT 9, "Times New Roman"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,70,33,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,126,33,50,14
    EDITTEXT        IDC_EDIT1,7,7,169,24,ES_AUTOHSCROLL | ES_OEMCONVERT
END

IDD_DIALOG_VORBIS DIALOG DISCARDABLE  0, 0, 197, 93
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Ogg Vorbis 設定ダイアログ"
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    CONTROL         "デフォルト設定",IDC_CHECK_DEFAULT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,4,0,56,10
    COMBOBOX        IDC_COMBO_MODE,25,13,123,56,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "タイトル",IDC_STATIC,4,49,23,8
    EDITTEXT        IDC_EDIT1,41,48,152,14,ES_AUTOHSCROLL | WS_DISABLED
    LTEXT           "アーティスト",IDC_STATIC,4,63,36,8
    EDITTEXT        IDC_EDIT2,41,76,152,14,ES_AUTOHSCROLL | WS_DISABLED
    LTEXT           "アルバム",IDC_STATIC,4,77,27,8
    EDITTEXT        IDC_EDIT3,41,62,152,14,ES_AUTOHSCROLL | WS_DISABLED
    CONTROL         "次の情報を書き込む",IDC_CHECK_USE_TAG,"Button",
                    BS_AUTOCHECKBOX | WS_DISABLED | WS_TABSTOP,4,32,73,10
    LTEXT           "モード",IDC_STATIC,4,16,18,8
    PUSHBUTTON      "OK",IDOK,152,2,40,12
    PUSHBUTTON      "キャンセル",IDCANCEL,152,30,40,12
    PUSHBUTTON      "適用",IDC_BUTTON_APPLY,152,14,40,11
END

IDD_DIALOG_GOGO DIALOG DISCARDABLE  0, 0, 346, 196
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "MP3(午後のこ〜だ) 設定ダイアログ"
FONT 8, "ＭＳ Ｐゴシック"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,173,179,50,14
    PUSHBUTTON      "キャンセル",IDCANCEL,285,179,50,14
    COMBOBOX        IDC_COMBO_OUTPUT_FORMAT,4,36,72,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "デフォルト設定",IDC_CHECK_DEFAULT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,4,0,80,10
    CONTROL         "コマンドラインオプション",IDC_CHECK_COMMANDLINE_OPTS,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,4,12,86,10
    EDITTEXT        IDC_EDIT_COMMANDLINE_OPTION,91,10,232,14,ES_AUTOHSCROLL
    COMBOBOX        IDC_COMBO_MPEG1_AUDIO_BITRATE,6,72,76,76,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_MPEG2_AUDIO_BITRATE,87,72,76,76,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_VBR_BITRATE_LOW,6,119,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "VBR - 最高",IDC_STATIC,86,110,55,8
    COMBOBOX        IDC_COMBO_VBR_BITRATE_HIGH,86,119,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "16KHz ローパスフィルタ",IDC_CHECK_16KHZ_LOW_PASS_FILTER,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,176,57,89,8
    CONTROL         "エンハンストローパスフィルタ",
                    IDC_CHECK_ENHANCED_LOW_PASS_FILTER,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,176,68,104,8
    EDITTEXT        IDC_EDIT_LPF_PARA1,200,77,28,12
    EDITTEXT        IDC_EDIT_LPF_PARA2,251,77,28,12
    LTEXT           "para1",IDC_STATIC,183,79,16,8
    LTEXT           "para2",IDC_STATIC,233,79,16,8
    COMBOBOX        IDC_COMBO_ENCODE_MODE,172,36,84,60,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "心理解析",IDC_CHECK_USEPSY,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,90,42,10
    CONTROL         "MMX",IDC_CHECK_CPUMMX,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,193,151,29,10
    CONTROL         "SSE",IDC_CHECK_CPUSSE,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,193,163,27,10
    CONTROL         "3DNOW",IDC_CHECK_CPU3DNOW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,227,151,39,10
    CONTROL         "E3DNOW",IDC_CHECK_CPUE3DNOW,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,227,163,40,10
    COMBOBOX        IDC_COMBO_EMPHASIS_TYPE,268,36,76,64,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_VBR,37,92,124,72,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    CONTROL         "VBR",IDC_CHECK_VBR,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,6,94,29,10
    CONTROL         "周波数",IDC_CHECK_OUTFREQ,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,88,28,35,8
    EDITTEXT        IDC_EDIT_OUTFREQ,88,36,72,12
    CONTROL         "J-Stereoでの Stereo/MS-Stereo の切替え制御",
                    IDC_CHECK_MSTHRESHOLD,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,103,167,8
    EDITTEXT        IDC_EDIT_MSTHRESHOLD_THRESHOLD,200,112,28,12
    EDITTEXT        IDC_EDIT_MSTHRESHOLD_MSPOWER,252,112,28,12
    LTEXT           "para1",IDC_STATIC,183,114,16,8
    LTEXT           "para2",IDC_STATIC,233,114,16,8
    CONTROL         "ベリファイ",IDC_CHECK_VERIFY,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,176,127,45,10
    PUSHBUTTON      "適用",IDC_BUTTON_APPLY,229,179,50,14
    CONTROL         "以下の命令を使う",IDC_CHECK_USE_CPU_OPTS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,176,140,68,8
    CONTROL         "MPEG1Audio",IDC_CHECK_MPEG1AUDIOBITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,64,80,8
    CONTROL         "MPEG2Audio",IDC_CHECK_MPEG2AUDIOBITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,87,64,80,8
    CONTROL         "VBR - 最低",IDC_CHECK_VBR_BITRATE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,6,110,67,8
    CONTROL         "出力フォーマット",IDC_CHECK_OUTPUT_FORMAT,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,4,28,64,8
    CONTROL         "モード",IDC_CHECK_ENCODE_MODE,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,172,28,34,8
    CONTROL         "エンファシス",IDC_CHECK_EMPHASIS_TYPE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,268,28,62,8
    GROUPBOX        "ビットレート",IDC_STATIC,2,55,167,86
    CONTROL         "CMOV",IDC_CHECK_CPUCMOV,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,276,140,39,9
    CONTROL         "EMMX",IDC_CHECK_CPUEMMX,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,276,152,39,9
    CONTROL         "SSE2",IDC_CHECK_CPUSSE2,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,276,164,39,8
END

IDD_DIALOG_ONE_LINE DIALOGEX 0, 0, 187, 63
STYLE DS_MODALFRAME | DS_3DLOOK | WS_POPUP | WS_CAPTION | WS_SYSMENU
EXSTYLE WS_EX_CONTROLPARENT
FONT 10, "ＭＳ Ｐゴシック", 0, 0, 0x1
BEGIN
    EDITTEXT        IDC_EDIT_ONE_LINE,4,16,180,14,ES_AUTOHSCROLL | WS_GROUP
    DEFPUSHBUTTON   "",IDC_BUTTON_1,32,48,50,14
    DEFPUSHBUTTON   "",IDC_BUTTON_2,84,48,50,14
    DEFPUSHBUTTON   "",IDC_BUTTON_3,136,48,50,14,WS_GROUP
    LTEXT           "",IDC_STATIC_TAIL,4,32,180,12
    LTEXT           "",IDC_STATIC_HEAD,4,0,180,12
END

IDD_PREF_SYN1 DIALOGEX 0, 0, 191, 177
STYLE DS_CONTROL | WS_CHILD | WS_CAPTION
FONT 9, "ＭＳ Ｐゴシック", 0, 0, 0x1
BEGIN
    LTEXT           "ポート 0",IDC_STATIC,6,27,24,12
    GROUPBOX        "MIDI IN デバイス",IDC_STATIC,0,0,174,102
    LTEXT           "ポート 1",IDC_STATIC,6,43,24,12
    LTEXT           "ポート 2",IDC_STATIC,6,57,24,12
    LTEXT           "ポート 3",IDC_STATIC,6,73,24,12
    COMBOBOX        IDC_COMBO_IDPORT0,36,27,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT1,36,42,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT2,36,57,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_IDPORT3,36,72,132,56,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "シンセスレッド優先度",IDC_STATIC,0,138,62,8
    COMBOBOX        IDC_COMBO_SYN_THREAD_PRIORITY,70,135,102,69,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "ポート数",IDC_STATIC,6,12,26,8
    COMBOBOX        IDC_COMBO_PORT_NUM,36,12,42,87,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "計算最大ミリ秒(推奨 500ms)",IDC_STATIC,0,156,87,8
    EDITTEXT        IDC_EDIT_SYN_SH_TIME,97,153,75,14,ES_AUTOHSCROLL
    LTEXT           "プロセス優先度",IDC_STATIC,0,124,46,8
    COMBOBOX        IDC_COMBO_PROCESS_PRIORITY,70,121,102,69,
                    CBS_DROPDOWNLIST | CBS_SORT | WS_VSCROLL | WS_TABSTOP
    CONTROL         "シンセを自動で開始する",IDC_CHECK_SYN_AUTOSTART,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,3,108,99,10
END

IDD_DIALOG_PREF DIALOG DISCARDABLE  0, 0, 231, 270
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "TiMidity++ Win32GUI 詳細設定"
FONT 9, "ＭＳ Ｐゴシック"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,54,255,50,14
    PUSHBUTTON      "ｷｬﾝｾﾙ",IDCANCEL,114,255,50,14
    PUSHBUTTON      "適用",IDC_BUTTON_APPLY,174,255,50,14
    CONTROL         "Tab1",IDC_TAB_MAIN,"SysTabControl32",0x0,0,0,231,252
END


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDM_MENU_MAIN MENU DISCARDABLE 
BEGIN
    POPUP "ファイル(&F)"
    BEGIN
        MENUITEM "ファイルを開く(&F)",          IDM_MFOPENFILE
        MENUITEM "フォルダを開く(&D)",          IDM_MFOPENDIR
        MENUITEM SEPARATOR
        MENUITEM "プレイリストを開く(&P)",      IDM_MFLOADPLAYLIST
        MENUITEM "プレイリストを保存(&S)",      IDM_MFSAVEPLAYLISTAS
        MENUITEM SEPARATOR
        MENUITEM "終了(&X)",                    IDM_MFEXIT
    END
    POPUP "設定(&C)"
    BEGIN
        MENUITEM "詳細設定(&P)",                IDM_SETTING
        MENUITEM SEPARATOR
        MENUITEM "設定読込(&L)",                IDM_MCLOADINIFILE
        MENUITEM "設定保存(&S)",                IDM_MCSAVEINIFILE
        MENUITEM SEPARATOR
        MENUITEM "cfg強制再読込(&F)",           IDM_FORCE_RELOAD
    END
    POPUP "ウィンドウ(&W)"
    BEGIN
        MENUITEM "プレイリスト(&L)",            IDM_MWPLAYLIST
        MENUITEM "トレーサ(&T)",                IDM_MWTRACER
        MENUITEM "ドキュメント(&D)",            IDM_MWDOCUMENT
        MENUITEM "WRD(&W)",                     IDM_MWWRDTRACER
        MENUITEM "コンソール(&C)",              IDM_MWCONSOLE
    END
    POPUP "ヘルプ(&H)"
    BEGIN
        MENUITEM "オンラインヘルプ(&O)",        IDM_MHONLINEHELP
        MENUITEM "コンフィグファイル詳解(&C)",  IDM_MHONLINEHELPCFG
        MENUITEM "バグ報告所(&B)",              IDM_MHBTS
        MENUITEM SEPARATOR
        MENUITEM "TiMidity++について(&T)",      IDM_MHTIMIDITY
        MENUITEM "バージョン情報(&V)",          IDM_MHVERSION
        MENUITEM SEPARATOR
        MENUITEM "補足(&S)",                    IDM_MHSUPPLEMENT
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO MOVEABLE PURE 
BEGIN
    IDD_DIALOG_CONSOLE, DIALOG
    BEGIN
        RIGHTMARGIN, 241
        BOTTOMMARGIN, 181
    END

    IDD_DIALOG_DOC, DIALOG
    BEGIN
        RIGHTMARGIN, 268
        BOTTOMMARGIN, 255
    END

    IDD_DIALOG_WRD, DIALOG
    BEGIN
        RIGHTMARGIN, 320
        BOTTOMMARGIN, 192
    END

    IDD_DIALOG_SIMPLE_LIST, DIALOG
    BEGIN
        RIGHTMARGIN, 241
        BOTTOMMARGIN, 187
    END

    IDD_PREF_PLAYER, DIALOG
    BEGIN
        BOTTOMMARGIN, 211
    END

    IDD_PREF_TIMIDITY1, DIALOG
    BEGIN
        RIGHTMARGIN, 146
        BOTTOMMARGIN, 164
    END

    IDD_PREF_TIMIDITY2, DIALOG
    BEGIN
        RIGHTMARGIN, 210
        BOTTOMMARGIN, 198
    END

    IDD_PREF_TIMIDITY3, DIALOG
    BEGIN
        RIGHTMARGIN, 190
        BOTTOMMARGIN, 138
    END

    IDD_PREF_TIMIDITY4, DIALOG
    BEGIN
        RIGHTMARGIN, 169
        BOTTOMMARGIN, 177
    END

    IDD_DIALOG_SEARCHBOX, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 176
        TOPMARGIN, 7
        BOTTOMMARGIN, 47
    END

    IDD_DIALOG_VORBIS, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 190
        TOPMARGIN, 7
        BOTTOMMARGIN, 86
    END

    IDD_DIALOG_GOGO, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 339
        TOPMARGIN, 7
        BOTTOMMARGIN, 189
    END

    IDD_DIALOG_ONE_LINE, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 180
        TOPMARGIN, 7
    END

    IDD_PREF_SYN1, DIALOG
    BEGIN
        RIGHTMARGIN, 184
        TOPMARGIN, 7
        BOTTOMMARGIN, 170
    END

    IDD_DIALOG_PREF, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 224
        TOPMARGIN, 7
        BOTTOMMARGIN, 263
    END
END
#endif    // APSTUDIO_INVOKED


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE MOVEABLE PURE 
BEGIN
    "w32g_res.h\0"
END

2 TEXTINCLUDE MOVEABLE PURE 
BEGIN
    "#define APSTUDIO_HIDDEN_SYMBOLS\r\n"
    "#include ""windows.h""\r\n"
    "#undef APSTUDIO_HIDDEN_SYMBOLS\r\n"
    "#include ""commctrl.h""\r\n"
    "#include ""w32g_rec.h""\r\n"
    "\0"
END

3 TEXTINCLUDE MOVEABLE PURE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 2,13,0,0
 PRODUCTVERSION 2,13,0,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004e4"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "\0"
            VALUE "FileDescription", "timw32g\0"
            VALUE "FileVersion", "2, 13, 0, 0\0"
            VALUE "InternalName", "timw32g\0"
            VALUE "LegalCopyright", "Copyright (C) 1999-2002 Masanao Izumo <mo@goice.co.jp>\r\nCopyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "OriginalFilename", "timw32g.exe\0"
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "TiMidity++ Win32GUI Version\0"
            VALUE "ProductVersion", "2, 13, 0, 0\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0, 1252
    END
END

#endif    // !_MAC


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_ICON_TIMIDITY       ICON    DISCARDABLE     "w32g_icon.ico"

/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDM_MHVERSION           "Version information of TiMidity++."
    IDM_SETTING             "Open the dialog of the modification of TiMidity parameters."
END

STRINGTABLE DISCARDABLE 
BEGIN
    IDM_MFOPENFILE          "Open midi files and add them to the playlist."
    IDM_MFOPENDIR           "Not Supported."
    IDM_MFLOADPLAYLIST      "Not Supported."
    IDM_MFSAVEPLAYLISTAS    "Not Supported."
    IDM_MFEXIT              "Exit TiMidity."
    IDM_MCLOADINIFILE       "Load ini file to apply TiMidity parameters."
    IDM_MCSAVEINIFILE       "Save ini file to save TiMidity parameters."
    IDM_MWPLAYLIST          "Open/Close playlist window to select midi files."
    IDM_MWTRACER            "Not Supported."
    IDM_MWDOCUMENT          "Not Supported."
    IDM_MWWRDTRACER         "Not Supported."
    IDM_MWSOUNDSPEC         "Not Supported."
    IDM_MWCONSOLE           "Open/Close Console Window."
    IDM_MWDEBUG             "For debugging by developpers of TiMidity."
    IDM_MHTOPIC             "Not Supported."
    IDM_MHHELP              "Not Supported."
END

STRINGTABLE DISCARDABLE 
BEGIN
    IDM_MHTIMIDITY          "Copying of TiMidity++."
END

#endif    // 日本語 resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

