

#ifndef __libOggFLAC_dll_h__
#define __libOggFLAC_dll_h__

#include "w32_libOggFLAC_dll_i.h"

/***************************************************************
   for header file
 ***************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif



/***************************************************************
   for definition of function type
 ***************************************************************/
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_new
typedef OggFLAC_API OggFLAC__StreamEncoder * (* libOggFLAC_func_OggFLAC__stream_encoder_new_t) ();
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_delete
typedef OggFLAC_API void (* libOggFLAC_func_OggFLAC__stream_encoder_delete_t) (OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_serial_number
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_serial_number_t) (OggFLAC__StreamEncoder *encoder, long serial_number);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_verify
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_verify_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_streamable_subset
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_streamable_subset_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_mid_side_stereo
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_do_mid_side_stereo_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_loose_mid_side_stereo
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_loose_mid_side_stereo_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_channels
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_channels_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_bits_per_sample
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_bits_per_sample_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_sample_rate
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_sample_rate_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_blocksize
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_blocksize_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_lpc_order
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_max_lpc_order_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_qlp_coeff_precision
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_qlp_coeff_precision_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_escape_coding
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_do_escape_coding_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_exhaustive_model_search
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_do_exhaustive_model_search_t) (OggFLAC__StreamEncoder *encoder, FLAC__bool value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_min_residual_partition_order
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_min_residual_partition_order_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_residual_partition_order
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_max_residual_partition_order_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_rice_parameter_search_dist
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_rice_parameter_search_dist_t) (OggFLAC__StreamEncoder *encoder, unsigned value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_total_samples_estimate
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_total_samples_estimate_t) (OggFLAC__StreamEncoder *encoder, FLAC__uint64 value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_metadata
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_metadata_t) (OggFLAC__StreamEncoder *encoder, FLAC__StreamMetadata **metadata, unsigned num_blocks);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_write_callback
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_write_callback_t) (OggFLAC__StreamEncoder *encoder, OggFLAC__StreamEncoderWriteCallback value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_client_data
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_set_client_data_t) (OggFLAC__StreamEncoder *encoder, void *value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_state
typedef OggFLAC_API OggFLAC__StreamEncoderState (* libOggFLAC_func_OggFLAC__stream_encoder_get_state_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_FLAC_stream_encoder_state
typedef OggFLAC_API FLAC__StreamEncoderState (* libOggFLAC_func_OggFLAC__stream_encoder_get_FLAC_stream_encoder_state_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_state
typedef OggFLAC_API FLAC__StreamDecoderState (* libOggFLAC_func_OggFLAC__stream_encoder_get_verify_decoder_state_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_error_stats
typedef OggFLAC_API void (* libOggFLAC_func_OggFLAC__stream_encoder_get_verify_decoder_error_stats_t) (const OggFLAC__StreamEncoder *encoder, FLAC__uint64 *absolute_sample, unsigned *frame_number, unsigned *channel, unsigned *sample, FLAC__int32 *expected, FLAC__int32 *got);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_verify_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_streamable_subset
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_streamable_subset_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_mid_side_stereo
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_do_mid_side_stereo_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_loose_mid_side_stereo
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_loose_mid_side_stereo_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_channels
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_channels_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_bits_per_sample
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_bits_per_sample_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_sample_rate
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_sample_rate_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_blocksize
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_blocksize_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_lpc_order
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_max_lpc_order_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_qlp_coeff_precision
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_qlp_coeff_precision_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_escape_coding
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_do_escape_coding_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_exhaustive_model_search
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_get_do_exhaustive_model_search_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_min_residual_partition_order
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_min_residual_partition_order_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_residual_partition_order
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_max_residual_partition_order_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_rice_parameter_search_dist
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_encoder_get_rice_parameter_search_dist_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_total_samples_estimate
typedef OggFLAC_API FLAC__uint64 (* libOggFLAC_func_OggFLAC__stream_encoder_get_total_samples_estimate_t) (const OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_init
typedef OggFLAC_API OggFLAC__StreamEncoderState (* libOggFLAC_func_OggFLAC__stream_encoder_init_t) (OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_finish
typedef OggFLAC_API void (* libOggFLAC_func_OggFLAC__stream_encoder_finish_t) (OggFLAC__StreamEncoder *encoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_process
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_process_t) (OggFLAC__StreamEncoder *encoder, const FLAC__int32 * const buffer[], unsigned samples);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_process_interleaved
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_encoder_process_interleaved_t) (OggFLAC__StreamEncoder *encoder, const FLAC__int32 buffer[], unsigned samples);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_new
typedef OggFLAC_API OggFLAC__StreamDecoder * (* libOggFLAC_func_OggFLAC__stream_decoder_new_t) ();
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_delete
typedef OggFLAC_API void (* libOggFLAC_func_OggFLAC__stream_decoder_delete_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_read_callback
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_read_callback_t) (OggFLAC__StreamDecoder *decoder, OggFLAC__StreamDecoderReadCallback value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_write_callback
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_write_callback_t) (OggFLAC__StreamDecoder *decoder, OggFLAC__StreamDecoderWriteCallback value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_callback
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_callback_t) (OggFLAC__StreamDecoder *decoder, OggFLAC__StreamDecoderMetadataCallback value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_error_callback
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_error_callback_t) (OggFLAC__StreamDecoder *decoder, OggFLAC__StreamDecoderErrorCallback value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_client_data
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_client_data_t) (OggFLAC__StreamDecoder *decoder, void *value);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_serial_number
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_serial_number_t) (OggFLAC__StreamDecoder *decoder, long serial_number);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_t) (OggFLAC__StreamDecoder *decoder, FLAC__MetadataType type);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_application
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_application_t) (OggFLAC__StreamDecoder *decoder, const FLAC__byte id[4]);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_all
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_all_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_t) (OggFLAC__StreamDecoder *decoder, FLAC__MetadataType type);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_application
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_application_t) (OggFLAC__StreamDecoder *decoder, const FLAC__byte id[4]);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_all
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_all_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_state
typedef OggFLAC_API OggFLAC__StreamDecoderState (* libOggFLAC_func_OggFLAC__stream_decoder_get_state_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_FLAC_stream_decoder_state
typedef OggFLAC_API FLAC__StreamDecoderState (* libOggFLAC_func_OggFLAC__stream_decoder_get_FLAC_stream_decoder_state_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channels
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_decoder_get_channels_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channel_assignment
typedef OggFLAC_API FLAC__ChannelAssignment (* libOggFLAC_func_OggFLAC__stream_decoder_get_channel_assignment_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_bits_per_sample
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_decoder_get_bits_per_sample_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_sample_rate
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_decoder_get_sample_rate_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_blocksize
typedef OggFLAC_API unsigned (* libOggFLAC_func_OggFLAC__stream_decoder_get_blocksize_t) (const OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_init
typedef OggFLAC_API OggFLAC__StreamDecoderState (* libOggFLAC_func_OggFLAC__stream_decoder_init_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_finish
typedef OggFLAC_API void (* libOggFLAC_func_OggFLAC__stream_decoder_finish_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_flush
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_flush_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_reset
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_reset_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_single
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_process_single_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_metadata
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_process_until_end_of_metadata_t) (OggFLAC__StreamDecoder *decoder);
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_stream
typedef OggFLAC_API FLAC__bool (* libOggFLAC_func_OggFLAC__stream_decoder_process_until_end_of_stream_t) (OggFLAC__StreamDecoder *decoder);
#endif


typedef struct libOggFLAC_dll_t_ {
  HANDLE __h_dll;
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_new
  libOggFLAC_func_OggFLAC__stream_encoder_new_t  OggFLAC__stream_encoder_new;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_delete
  libOggFLAC_func_OggFLAC__stream_encoder_delete_t  OggFLAC__stream_encoder_delete;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_serial_number
  libOggFLAC_func_OggFLAC__stream_encoder_set_serial_number_t  OggFLAC__stream_encoder_set_serial_number;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_verify
  libOggFLAC_func_OggFLAC__stream_encoder_set_verify_t  OggFLAC__stream_encoder_set_verify;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_streamable_subset
  libOggFLAC_func_OggFLAC__stream_encoder_set_streamable_subset_t  OggFLAC__stream_encoder_set_streamable_subset;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_mid_side_stereo
  libOggFLAC_func_OggFLAC__stream_encoder_set_do_mid_side_stereo_t  OggFLAC__stream_encoder_set_do_mid_side_stereo;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_loose_mid_side_stereo
  libOggFLAC_func_OggFLAC__stream_encoder_set_loose_mid_side_stereo_t  OggFLAC__stream_encoder_set_loose_mid_side_stereo;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_channels
  libOggFLAC_func_OggFLAC__stream_encoder_set_channels_t  OggFLAC__stream_encoder_set_channels;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_bits_per_sample
  libOggFLAC_func_OggFLAC__stream_encoder_set_bits_per_sample_t  OggFLAC__stream_encoder_set_bits_per_sample;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_sample_rate
  libOggFLAC_func_OggFLAC__stream_encoder_set_sample_rate_t  OggFLAC__stream_encoder_set_sample_rate;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_blocksize
  libOggFLAC_func_OggFLAC__stream_encoder_set_blocksize_t  OggFLAC__stream_encoder_set_blocksize;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_lpc_order
  libOggFLAC_func_OggFLAC__stream_encoder_set_max_lpc_order_t  OggFLAC__stream_encoder_set_max_lpc_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_qlp_coeff_precision
  libOggFLAC_func_OggFLAC__stream_encoder_set_qlp_coeff_precision_t  OggFLAC__stream_encoder_set_qlp_coeff_precision;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search
  libOggFLAC_func_OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search_t  OggFLAC__stream_encoder_set_do_qlp_coeff_prec_search;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_escape_coding
  libOggFLAC_func_OggFLAC__stream_encoder_set_do_escape_coding_t  OggFLAC__stream_encoder_set_do_escape_coding;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_do_exhaustive_model_search
  libOggFLAC_func_OggFLAC__stream_encoder_set_do_exhaustive_model_search_t  OggFLAC__stream_encoder_set_do_exhaustive_model_search;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_min_residual_partition_order
  libOggFLAC_func_OggFLAC__stream_encoder_set_min_residual_partition_order_t  OggFLAC__stream_encoder_set_min_residual_partition_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_max_residual_partition_order
  libOggFLAC_func_OggFLAC__stream_encoder_set_max_residual_partition_order_t  OggFLAC__stream_encoder_set_max_residual_partition_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_rice_parameter_search_dist
  libOggFLAC_func_OggFLAC__stream_encoder_set_rice_parameter_search_dist_t  OggFLAC__stream_encoder_set_rice_parameter_search_dist;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_total_samples_estimate
  libOggFLAC_func_OggFLAC__stream_encoder_set_total_samples_estimate_t  OggFLAC__stream_encoder_set_total_samples_estimate;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_metadata
  libOggFLAC_func_OggFLAC__stream_encoder_set_metadata_t  OggFLAC__stream_encoder_set_metadata;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_write_callback
  libOggFLAC_func_OggFLAC__stream_encoder_set_write_callback_t  OggFLAC__stream_encoder_set_write_callback;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_set_client_data
  libOggFLAC_func_OggFLAC__stream_encoder_set_client_data_t  OggFLAC__stream_encoder_set_client_data;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_state
  libOggFLAC_func_OggFLAC__stream_encoder_get_state_t  OggFLAC__stream_encoder_get_state;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_FLAC_stream_encoder_state
  libOggFLAC_func_OggFLAC__stream_encoder_get_FLAC_stream_encoder_state_t  OggFLAC__stream_encoder_get_FLAC_stream_encoder_state;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_state
  libOggFLAC_func_OggFLAC__stream_encoder_get_verify_decoder_state_t  OggFLAC__stream_encoder_get_verify_decoder_state;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify_decoder_error_stats
  libOggFLAC_func_OggFLAC__stream_encoder_get_verify_decoder_error_stats_t  OggFLAC__stream_encoder_get_verify_decoder_error_stats;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_verify
  libOggFLAC_func_OggFLAC__stream_encoder_get_verify_t  OggFLAC__stream_encoder_get_verify;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_streamable_subset
  libOggFLAC_func_OggFLAC__stream_encoder_get_streamable_subset_t  OggFLAC__stream_encoder_get_streamable_subset;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_mid_side_stereo
  libOggFLAC_func_OggFLAC__stream_encoder_get_do_mid_side_stereo_t  OggFLAC__stream_encoder_get_do_mid_side_stereo;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_loose_mid_side_stereo
  libOggFLAC_func_OggFLAC__stream_encoder_get_loose_mid_side_stereo_t  OggFLAC__stream_encoder_get_loose_mid_side_stereo;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_channels
  libOggFLAC_func_OggFLAC__stream_encoder_get_channels_t  OggFLAC__stream_encoder_get_channels;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_bits_per_sample
  libOggFLAC_func_OggFLAC__stream_encoder_get_bits_per_sample_t  OggFLAC__stream_encoder_get_bits_per_sample;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_sample_rate
  libOggFLAC_func_OggFLAC__stream_encoder_get_sample_rate_t  OggFLAC__stream_encoder_get_sample_rate;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_blocksize
  libOggFLAC_func_OggFLAC__stream_encoder_get_blocksize_t  OggFLAC__stream_encoder_get_blocksize;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_lpc_order
  libOggFLAC_func_OggFLAC__stream_encoder_get_max_lpc_order_t  OggFLAC__stream_encoder_get_max_lpc_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_qlp_coeff_precision
  libOggFLAC_func_OggFLAC__stream_encoder_get_qlp_coeff_precision_t  OggFLAC__stream_encoder_get_qlp_coeff_precision;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search
  libOggFLAC_func_OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search_t  OggFLAC__stream_encoder_get_do_qlp_coeff_prec_search;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_escape_coding
  libOggFLAC_func_OggFLAC__stream_encoder_get_do_escape_coding_t  OggFLAC__stream_encoder_get_do_escape_coding;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_do_exhaustive_model_search
  libOggFLAC_func_OggFLAC__stream_encoder_get_do_exhaustive_model_search_t  OggFLAC__stream_encoder_get_do_exhaustive_model_search;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_min_residual_partition_order
  libOggFLAC_func_OggFLAC__stream_encoder_get_min_residual_partition_order_t  OggFLAC__stream_encoder_get_min_residual_partition_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_max_residual_partition_order
  libOggFLAC_func_OggFLAC__stream_encoder_get_max_residual_partition_order_t  OggFLAC__stream_encoder_get_max_residual_partition_order;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_rice_parameter_search_dist
  libOggFLAC_func_OggFLAC__stream_encoder_get_rice_parameter_search_dist_t  OggFLAC__stream_encoder_get_rice_parameter_search_dist;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_get_total_samples_estimate
  libOggFLAC_func_OggFLAC__stream_encoder_get_total_samples_estimate_t  OggFLAC__stream_encoder_get_total_samples_estimate;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_init
  libOggFLAC_func_OggFLAC__stream_encoder_init_t  OggFLAC__stream_encoder_init;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_finish
  libOggFLAC_func_OggFLAC__stream_encoder_finish_t  OggFLAC__stream_encoder_finish;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_process
  libOggFLAC_func_OggFLAC__stream_encoder_process_t  OggFLAC__stream_encoder_process;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_encoder_process_interleaved
  libOggFLAC_func_OggFLAC__stream_encoder_process_interleaved_t  OggFLAC__stream_encoder_process_interleaved;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_new
  libOggFLAC_func_OggFLAC__stream_decoder_new_t  OggFLAC__stream_decoder_new;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_delete
  libOggFLAC_func_OggFLAC__stream_decoder_delete_t  OggFLAC__stream_decoder_delete;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_read_callback
  libOggFLAC_func_OggFLAC__stream_decoder_set_read_callback_t  OggFLAC__stream_decoder_set_read_callback;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_write_callback
  libOggFLAC_func_OggFLAC__stream_decoder_set_write_callback_t  OggFLAC__stream_decoder_set_write_callback;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_callback
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_callback_t  OggFLAC__stream_decoder_set_metadata_callback;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_error_callback
  libOggFLAC_func_OggFLAC__stream_decoder_set_error_callback_t  OggFLAC__stream_decoder_set_error_callback;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_client_data
  libOggFLAC_func_OggFLAC__stream_decoder_set_client_data_t  OggFLAC__stream_decoder_set_client_data;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_serial_number
  libOggFLAC_func_OggFLAC__stream_decoder_set_serial_number_t  OggFLAC__stream_decoder_set_serial_number;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_t  OggFLAC__stream_decoder_set_metadata_respond;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_application
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_application_t  OggFLAC__stream_decoder_set_metadata_respond_application;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_respond_all
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_respond_all_t  OggFLAC__stream_decoder_set_metadata_respond_all;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_t  OggFLAC__stream_decoder_set_metadata_ignore;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_application
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_application_t  OggFLAC__stream_decoder_set_metadata_ignore_application;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_set_metadata_ignore_all
  libOggFLAC_func_OggFLAC__stream_decoder_set_metadata_ignore_all_t  OggFLAC__stream_decoder_set_metadata_ignore_all;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_state
  libOggFLAC_func_OggFLAC__stream_decoder_get_state_t  OggFLAC__stream_decoder_get_state;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_FLAC_stream_decoder_state
  libOggFLAC_func_OggFLAC__stream_decoder_get_FLAC_stream_decoder_state_t  OggFLAC__stream_decoder_get_FLAC_stream_decoder_state;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channels
  libOggFLAC_func_OggFLAC__stream_decoder_get_channels_t  OggFLAC__stream_decoder_get_channels;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_channel_assignment
  libOggFLAC_func_OggFLAC__stream_decoder_get_channel_assignment_t  OggFLAC__stream_decoder_get_channel_assignment;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_bits_per_sample
  libOggFLAC_func_OggFLAC__stream_decoder_get_bits_per_sample_t  OggFLAC__stream_decoder_get_bits_per_sample;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_sample_rate
  libOggFLAC_func_OggFLAC__stream_decoder_get_sample_rate_t  OggFLAC__stream_decoder_get_sample_rate;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_get_blocksize
  libOggFLAC_func_OggFLAC__stream_decoder_get_blocksize_t  OggFLAC__stream_decoder_get_blocksize;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_init
  libOggFLAC_func_OggFLAC__stream_decoder_init_t  OggFLAC__stream_decoder_init;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_finish
  libOggFLAC_func_OggFLAC__stream_decoder_finish_t  OggFLAC__stream_decoder_finish;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_flush
  libOggFLAC_func_OggFLAC__stream_decoder_flush_t  OggFLAC__stream_decoder_flush;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_reset
  libOggFLAC_func_OggFLAC__stream_decoder_reset_t  OggFLAC__stream_decoder_reset;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_single
  libOggFLAC_func_OggFLAC__stream_decoder_process_single_t  OggFLAC__stream_decoder_process_single;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_metadata
  libOggFLAC_func_OggFLAC__stream_decoder_process_until_end_of_metadata_t  OggFLAC__stream_decoder_process_until_end_of_metadata;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__stream_decoder_process_until_end_of_stream
  libOggFLAC_func_OggFLAC__stream_decoder_process_until_end_of_stream_t  OggFLAC__stream_decoder_process_until_end_of_stream;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__StreamEncoderStateString
  OggFLAC_API const char * const*   OggFLAC__StreamEncoderStateString;
#endif
#ifndef IGNORE_libOggFLAC_OggFLAC__StreamDecoderStateString
  OggFLAC_API const char * const*   OggFLAC__StreamDecoderStateString;
#endif
} libOggFLAC_dll_t;

extern libOggFLAC_dll_t *load_libOggFLAC_dll ( char *path );
extern void free_libOggFLAC_dll ( libOggFLAC_dll_t *dll );


#if defined(__cplusplus)
}  /* extern "C" { */
#endif

/***************************************************************/

#endif  /* __libOggFLAC_dll_h__ */

